#include <cmath>
#include <GL/gl.h>

void drawCylinderCone(double radiusBase, double radiusTop, double height, double slices){
    height*=0.5 ;
    double full = M_PI*2 ;
    double normalMag = 1 ;
    double reflectivity=0.025 ;
	
    glBegin(GL_LINE_LOOP) ;
	int i ;
    for ( i=1 ; i<slices+1; ++i ){
        double tf = (full/slices)*(i-1);
        double xf = cos(tf);
        double zf = sin(tf);
        
        double ts = (full/slices)*i;
        double xs = cos(ts);
        double zs = sin(ts);
            
        glNormal3f( 1, 0, 0 );
		
        double v1[] = {radiusBase*xf,-height,radiusBase*zf};
        double v2[] = {radiusBase*xs,-height,radiusBase*zs};
        double v3[] = {radiusTop*xs,height,radiusTop*zs};
        double v4[] = {radiusTop*xf,height,radiusTop*zf};
        
        double n[] = {  (v1[0]+v2[0]+v3[0]+v4[0])/4, (v1[1]+v2[1]+v3[1]+v4[1])/4, (v1[2]+v2[2]+v3[2]+v4[2])/4 };
        double nAbs = sqrt( (n[0]*n[0])+(n[1]*n[1])+(n[2]*n[2]) );
        double n2[] = { n[0]/nAbs*reflectivity, n[1]/nAbs*reflectivity, n[2]/nAbs*reflectivity };
        
        //glNormal3d(n[0],n[1],n[2]);
        glVertex3d(v1[0],v1[1],v1[2]);
        glVertex3d(v2[0],v2[1],v2[2]);
        glVertex3d(v3[0],v3[1],v3[2]);
        glVertex3d(v4[0],v4[1],v4[2]);
        glVertex3d(v1[0],v1[1],v1[2]);
	}
        
    glEnd();
}

double sphereRad( double radius, double slices, double i){
    double y = i/slices ;
    double sqRad = 1 - (y*y) ;
    double rad = sqrt(sqRad)*radius ;
    return rad ;
}


void drawCap(double radius, double slices, double posneg){
    double height = radius/slices ;
    glPushMatrix() ;
    glScaled(1,posneg,1) ;
	int i;
    for (i=1; i<slices; ++i){
        double radBase = sphereRad(radius,slices,i-1);
        double radUp = sphereRad(radius,slices,i);
        drawCylinderCone(radBase,radUp,height,slices);
        glTranslated(0,height,0);
	}
    glPopMatrix();
}


void drawCapsule(double radius,double height,int slices){
    glPushMatrix();
    double half = height/2;
    glTranslated(0,half,0);
    drawCap(radius,slices,1);
    
    glTranslated(0,-half,0);
    drawCylinderCone(radius,radius,height,slices);
    
    glTranslated(0,-half,0);
    drawCap(radius,slices,-1);

    glPopMatrix();
}

void drawWireCapsule(double radius,double height,int rez){
    glPushAttrib(GL_CURRENT_BIT );
    glBegin(GL_LINE_STRIP);

    double slice= (M_PI*2.0)/(float)rez;
    
	double t, x, y, z ;
	
	int i;
    for (i=0; i<=rez; ++i){
        t = (double)i*slice;
        x = radius*cos(t);
        z = radius*sin(t);
        glVertex3f(x,z,-height/2.0);
	}
    glEnd();

    glBegin(GL_LINE_STRIP);
    for (i=0; i<=rez; ++i){
        t = (double)i*slice;
        x = radius*cos(t);
        z = radius*sin(t);
        glVertex3f(x,z,height/2.0);
	}
    glEnd();
    
    glBegin(GL_LINE_STRIP);
    for (i=0; i<=(rez/2); ++i){
        t = (double)i*slice;
        x = radius*cos(t);
        y = radius*sin(t);
        glVertex3f(x,0,y+(height/2));
	}
    glEnd();
    
    glBegin(GL_LINE_STRIP);
    for (i=0; i<=(rez/2); ++i){
        t = (double)i*slice;
        x = radius*cos(t);
        y = radius*sin(t);
        glVertex3f(x,0,-y-(height/2));
	}
    glEnd();
    
    glBegin(GL_LINE_STRIP);
    for (i=0; i<=(rez/2); ++i){
        t = (double)i*slice;
        z = radius*cos(t);
        y = radius*sin(t);
        glVertex3f(0,z,-y-(height/2));
	}
    glEnd();
    
    glBegin(GL_LINE_STRIP);
    for (i=0; i<=(rez/2); ++i){
        t = (double)i*slice;
        z = radius*cos(t);
        y = radius*sin(t);
        glVertex3f(0,z,y+(height/2));
	}
    glEnd();
    
    glBegin(GL_LINES);
    slice = M_PI / 2;
    for (i=0; i<=4; ++i){
        t = (double)i*slice;
        x = radius*cos(t);
        z = radius*sin(t);
        glVertex3d(x,z,-height/2.0);
        glVertex3d(x,z,height/2.0);
	}
    glEnd();
    
    glPopAttrib();
}

void drawWireBox( double halfWidth, double halfHeight, double halfDepth ){
	glBegin(GL_LINE_LOOP);
		///Front
		glVertex3d(-halfWidth, -halfHeight, -halfDepth);
		glVertex3d(-halfWidth, halfHeight, -halfDepth);
		glVertex3d(halfWidth, halfHeight, -halfDepth);
		glVertex3d(halfWidth, -halfHeight, -halfDepth);
	glEnd();
	glBegin(GL_LINE_LOOP);
		///Back
		glVertex3d(-halfWidth, -halfHeight, halfDepth);
		glVertex3d(-halfWidth, halfHeight, halfDepth);
		glVertex3d(halfWidth, halfHeight, halfDepth);
		glVertex3d(halfWidth, -halfHeight, halfDepth);
	glEnd();
	glBegin(GL_LINES);
		///Left
		glVertex3d(-halfWidth, -halfHeight, -halfDepth);
		glVertex3d(-halfWidth, -halfHeight, halfDepth);
		glVertex3d(-halfWidth, halfHeight, -halfDepth);
		glVertex3d(-halfWidth, halfHeight, halfDepth);
		
		glVertex3d(halfWidth, -halfHeight, -halfDepth);
		glVertex3d(halfWidth, -halfHeight, halfDepth);
		glVertex3d(halfWidth, halfHeight, -halfDepth);
		glVertex3d(halfWidth, halfHeight, halfDepth);
	glEnd();
}

void drawForce(){
	double TipHeight = 2 ;
	double BaseHeight = 4 ;
	double radius = 1 ;
	glTranslated(0,-1,0);
	drawCylinderCone(radius, 0.001, TipHeight, 10);
	glTranslated(0,-(TipHeight/2)-(BaseHeight/2),0);
	drawCylinderCone(radius/2, radius/2, BaseHeight, 5);
}

