/*c

from luCommon import *

class nodeInfo:
	name = "luBox"
	typeid = "0x80000"
	compoundAttributes =	{ "dimensions" : ["mass","height", "width", "depth"] }

genCPP(nodeInfo)
c*/
#include<luBox.h>
#define NODENAME luBox 
//e

#include <maya/MColor.h>
#include <glDrawUtils.h>

/*c
genStaticMembers(nodeInfo)
c*/
const MTypeId NODENAME::typeId( 0x80000 );
const MString NODENAME::typeName( "luBox" );
MObject NODENAME::m_aDimensions ;
MObject NODENAME::m_aDimMass ;
MObject NODENAME::m_aDimHeight ;
MObject NODENAME::m_aDimWidth ;
MObject NODENAME::m_aDimDepth ;
//e

void NODENAME::init(){
	MFnDependencyNode fc(thisMObject());
	
	MPlug pPlug = fc.findPlug("lpx") ;
	pPlug.setChannelBox(false);

	pPlug = fc.findPlug("lpy") ;
	pPlug.setChannelBox(false);	

	pPlug = fc.findPlug("lpz") ;
	pPlug.setChannelBox(false);
	
	pPlug = fc.findPlug("lsx") ;
	pPlug.setChannelBox(false);
	
	pPlug = fc.findPlug("lsy") ;
	pPlug.setChannelBox(false);
	
	pPlug = fc.findPlug("lsz") ;
	pPlug.setChannelBox(false);
}


void NODENAME::draw( M3dView& view, const MDagPath& DGpath, M3dView::DisplayStyle style, M3dView::DisplayStatus status)
{
	if(subInit){
		init();
		subInit = false ;
	}
	
	MColor col = colorRGB( status );
	
	double Height = getDimHeight();
	double Width = getDimWidth() ;	
	double Depth = getDimDepth() ;	

	// think glPushMatrix()
	view.beginGL();

	// this makes a copy of the current openGL settings so that anything
	// we change will not affect anything else maya draws afterwards.
	glPushAttrib( GL_CURRENT_BIT );
	glLineWidth(2);
	
	if(status == M3dView::kLead)
			glColor4f(0,1,0,0.5f);
		else
			glColor4f(col.r,col.g,col.b,0.5f);
	
	glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
		drawWireBox(Height,Width,Depth);
	glDisable(GL_BLEND);
	glLineWidth(1);
	// restore the old openGL settings
	glPopAttrib();

	// think glPopMatrix()
	view.endGL();
}

//------------------------------------------------------------------------
/// \brief	this function can tell maya whether the locator node has a volume
/// \return	true if bounded, false otherwise.
///
bool NODENAME::isBounded() const {
	return true;
}

//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox NODENAME::boundingBox() const
{
	double Height = getDimHeight();
	double Width = getDimWidth() ;	
	double Depth = getDimDepth() ;	

	MBoundingBox bbox;
	
	double halfHeight = Height ;
	double halfWidth = Width ;
	double halfDepth = Depth ;
	
	// simply expand the bounding box to contain the points used

	
	bbox.expand( MPoint( 0.0f, halfWidth, 0.0f ) );
	bbox.expand( MPoint( 0.0f,-halfWidth, 0.0f ) );

	bbox.expand( MPoint( halfHeight, 0.0f, 0.0f ) );
	bbox.expand( MPoint( -halfHeight, 0.0f, 0.0f ) );

	bbox.expand( MPoint( 0.0f, 0.0f, halfDepth ) );
	bbox.expand( MPoint( 0.0f, 0.0f,-halfDepth ) );

	return bbox;
}

void* NODENAME::creator()
{
	NODENAME *newLoc = new NODENAME();
	newLoc->subInit = true ;
	return newLoc ;
}

MStatus NODENAME::initialize()
{
/*c
genAttributeInitialisers( nodeInfo, setKeyable=1 )
c*/
        MFnCompoundAttribute cAttr_Dimensions;
        m_aDimensions=cAttr_Dimensions.create("dimensions","dimensions");
        cAttr_Dimensions.setStorable(true);
        cAttr_Dimensions.setKeyable(true);
        cAttr_Dimensions.setReadable(true);

	MFnNumericAttribute nAttr_Dimensions;
        m_aDimMass = nAttr_Dimensions.create("mass","mass",MFnNumericData::kDouble);
        nAttr_Dimensions.setStorable(true);
        nAttr_Dimensions.setKeyable(true);
        nAttr_Dimensions.setHidden(false);
        nAttr_Dimensions.setChannelBox(true);
        nAttr_Dimensions.setReadable(true);
        nAttr_Dimensions.setDefault(1.0);
        cAttr_Dimensions.addChild(m_aDimMass);

        m_aDimHeight = nAttr_Dimensions.create("height","height",MFnNumericData::kDouble);
        nAttr_Dimensions.setStorable(true);
        nAttr_Dimensions.setKeyable(true);
        nAttr_Dimensions.setHidden(false);
        nAttr_Dimensions.setChannelBox(true);
        nAttr_Dimensions.setReadable(true);
        nAttr_Dimensions.setDefault(1.0);
        cAttr_Dimensions.addChild(m_aDimHeight);

        m_aDimWidth = nAttr_Dimensions.create("width","width",MFnNumericData::kDouble);
        nAttr_Dimensions.setStorable(true);
        nAttr_Dimensions.setKeyable(true);
        nAttr_Dimensions.setHidden(false);
        nAttr_Dimensions.setChannelBox(true);
        nAttr_Dimensions.setReadable(true);
        nAttr_Dimensions.setDefault(1.0);
        cAttr_Dimensions.addChild(m_aDimWidth);

        m_aDimDepth = nAttr_Dimensions.create("depth","depth",MFnNumericData::kDouble);
        nAttr_Dimensions.setStorable(true);
        nAttr_Dimensions.setKeyable(true);
        nAttr_Dimensions.setHidden(false);
        nAttr_Dimensions.setChannelBox(true);
        nAttr_Dimensions.setReadable(true);
        nAttr_Dimensions.setDefault(1.0);
        cAttr_Dimensions.addChild(m_aDimDepth);

	addAttribute(m_aDimensions) ;
        addAttribute(m_aDimMass) ;
        addAttribute(m_aDimHeight) ;
        addAttribute(m_aDimWidth) ;
        addAttribute(m_aDimDepth) ;
//e
	return MS::kSuccess;
}

/*c
genAttributeGetters(nodeInfo)
c*/
double NODENAME::getDimMass() const {
	MPlug dataP(thisMObject(),m_aDimMass);
	double dataD;
	dataP.getValue(dataD);
	return dataD;
}
double NODENAME::getDimHeight() const {
	MPlug dataP(thisMObject(),m_aDimHeight);
	double dataD;
	dataP.getValue(dataD);
	return dataD;
}
double NODENAME::getDimWidth() const {
	MPlug dataP(thisMObject(),m_aDimWidth);
	double dataD;
	dataP.getValue(dataD);
	return dataD;
}
double NODENAME::getDimDepth() const {
	MPlug dataP(thisMObject(),m_aDimDepth);
	double dataD;
	dataP.getValue(dataD);
	return dataD;
}
//e

#undef NODENAME

