/*c

from luCommon import *

class nodeInfo:
	name = "luCapsule"
	typeid = "0x70000"
	compoundAttributes =	{ "dimensions" : ["mass", "radius", "length"]}

genCPP(nodeInfo)
c*/
#include<luCapsule.h>
#define NODENAME luCapsule 
//e

#include <maya/MColor.h>
#include <glDrawUtils.h>

/*c
genStaticMembers(nodeInfo)
c*/
const MTypeId NODENAME::typeId( 0x70000 );
const MString NODENAME::typeName( "luCapsule" );
MObject NODENAME::m_aDimensions ;
MObject NODENAME::m_aDimMass ;
MObject NODENAME::m_aDimRadius ;
MObject NODENAME::m_aDimLength ;
//e

void NODENAME::init(){
	MFnDependencyNode fc(thisMObject());
	
	MPlug pPlug = fc.findPlug("lpx") ;
	pPlug.setChannelBox(false);

	pPlug = fc.findPlug("lpy") ;
	pPlug.setChannelBox(false);	

	pPlug = fc.findPlug("lpz") ;
	pPlug.setChannelBox(false);
	
	pPlug = fc.findPlug("lsx") ;
	pPlug.setChannelBox(false);
	
	pPlug = fc.findPlug("lsy") ;
	pPlug.setChannelBox(false);
	
	pPlug = fc.findPlug("lsz") ;
	pPlug.setChannelBox(false);
}


void NODENAME::draw( M3dView& view, const MDagPath& DGpath, M3dView::DisplayStyle style, M3dView::DisplayStatus status)
{
	
	if(subInit){
		init();
		subInit = false ;
	}
	
	MColor col = colorRGB( status );
	
	double length = getDimLength();
	double radius = getDimRadius();

	// think glPushMatrix()
	view.beginGL();

	// this makes a copy of the current openGL settings so that anything
	// we change will not affect anything else maya draws afterwards.
	glPushAttrib( GL_CURRENT_BIT );
	glLineWidth(2);
	
	if(status == M3dView::kLead)
			glColor4f(0,1,0,0.5f);
		else
			glColor4f(col.r,col.g,col.b,0.5f);
	
	glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
		glPushMatrix();
			drawWireCapsule(radius,length,50);
		glPopMatrix();
	glDisable(GL_BLEND);
	glLineWidth(1);
	// restore the old openGL settings
	glPopAttrib();

	// think glPopMatrix()
	view.endGL();
}

//------------------------------------------------------------------------
/// \brief	this function can tell maya whether the locator node has a volume
/// \return	true if bounded, false otherwise.
///
bool NODENAME::isBounded() const {
	return true;
}

//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox NODENAME::boundingBox() const
{
	double length = getDimLength();
	double radius = getDimRadius();
	
	MBoundingBox bbox;
	
	double halfHeight = (length/2)+radius ;
	
	// simply expand the bounding box to contain the points used
	bbox.expand( MPoint( 0.0f, 0.0f,  static_cast<float>(halfHeight) ) );
	bbox.expand( MPoint( 0.0f, 0.0f, -static_cast<float>(halfHeight) ) );
	
	bbox.expand( MPoint( radius, 0.0f, 0.0f ) );
	bbox.expand( MPoint( -radius, 0.0f, 0.0f ) );
	
	bbox.expand( MPoint( 0.0f, radius, 0.0f ) );
	bbox.expand( MPoint( 0.0f, -radius, 0.0f ) );

	return bbox;
}

void* NODENAME::creator()
{
	NODENAME *newLoc = new NODENAME();
	newLoc->subInit = true ;
	return newLoc ;
}

MStatus NODENAME::initialize()
{
/*c
genAttributeInitialisers( nodeInfo )
c*/
        MFnCompoundAttribute cAttr_Dimensions;
        m_aDimensions=cAttr_Dimensions.create("dimensions","dimensions");
        cAttr_Dimensions.setStorable(true);
        cAttr_Dimensions.setKeyable(false);
        cAttr_Dimensions.setReadable(true);

	MFnNumericAttribute nAttr_Dimensions;
        m_aDimMass = nAttr_Dimensions.create("mass","mass",MFnNumericData::kDouble);
        nAttr_Dimensions.setStorable(true);
        nAttr_Dimensions.setKeyable(false);
        nAttr_Dimensions.setHidden(false);
        nAttr_Dimensions.setChannelBox(true);
        nAttr_Dimensions.setReadable(true);
        nAttr_Dimensions.setDefault(1.0);
        cAttr_Dimensions.addChild(m_aDimMass);

        m_aDimRadius = nAttr_Dimensions.create("radius","radius",MFnNumericData::kDouble);
        nAttr_Dimensions.setStorable(true);
        nAttr_Dimensions.setKeyable(false);
        nAttr_Dimensions.setHidden(false);
        nAttr_Dimensions.setChannelBox(true);
        nAttr_Dimensions.setReadable(true);
        nAttr_Dimensions.setDefault(1.0);
        cAttr_Dimensions.addChild(m_aDimRadius);

        m_aDimLength = nAttr_Dimensions.create("length","length",MFnNumericData::kDouble);
        nAttr_Dimensions.setStorable(true);
        nAttr_Dimensions.setKeyable(false);
        nAttr_Dimensions.setHidden(false);
        nAttr_Dimensions.setChannelBox(true);
        nAttr_Dimensions.setReadable(true);
        nAttr_Dimensions.setDefault(1.0);
        cAttr_Dimensions.addChild(m_aDimLength);

	addAttribute(m_aDimensions) ;
        addAttribute(m_aDimMass) ;
        addAttribute(m_aDimRadius) ;
        addAttribute(m_aDimLength) ;
//e
	return MS::kSuccess;
}

/*c
genAttributeGetters(nodeInfo)
c*/
double NODENAME::getDimMass() const {
	MPlug dataP(thisMObject(),m_aDimMass);
	double dataD;
	dataP.getValue(dataD);
	return dataD;
}
double NODENAME::getDimRadius() const {
	MPlug dataP(thisMObject(),m_aDimRadius);
	double dataD;
	dataP.getValue(dataD);
	return dataD;
}
double NODENAME::getDimLength() const {
	MPlug dataP(thisMObject(),m_aDimLength);
	double dataD;
	dataP.getValue(dataD);
	return dataD;
}
//e

#undef NODENAME

