#Modules needed my the current module. These can be thought of as includes and libraries in one thing.
import luyaODE as luya ; reload(luya)
from maya.mel import *
from mayaUtils import *
from luyaUtils import *

class jointPID:
    def __init__( self, w, joint, kP, kI, kD ):
        self.kP = kP
        self.kI = kI
        self.kD = kD
        self.w = w
        
        self.lastErrorX = 0
        self.lastErrorY = 0
        self.lastErrorZ =0
        
        self.joint = joint
        
        #del joint.motor
        
    def __call__(self):
        joint = self.joint
        #Retrieve the Current Direction of the Inverse Pendulum.
        Current = Translate( joint.child ) - Translate( joint.name )
        
        #Retrieve the Direction of where we want to be.
        Reference = MVector( 0, 1, 0 )
        
        # The Axis is the cross product of the Reference and Current vectors.
        # when Current moves to the other side of Reference the axis will flip,
        # this will flip the direction of rotation. A side effect of the design.
        # All we need to worry about is the AngularVelocity.
        Axis = ( Reference ^ Current ).normal()
        
        # The current error. If this is 0 then we are in the right place.
        Error = Current.angle( Reference )
        
        # Using a quaternion allows us to perform an neat Axis Angle rotation.
        EError = MQuaternion( Error, Axis ) . asEulerRotation()
        
        # Both of these constants have to be tweaked by hand
        kP = self.kP     # The Proportional Constant
        kD = self.kD # The Derivative Constant
        
        #Performing the PID Controll all all 3 Axis of Rotation
        Px = EError.x
        Dx = EError.x - self.lastErrorX
        SolutionX = kP*Px + kD*Dx
        
        Py = EError.y
        Dy = EError.y - self.lastErrorY
        SolutionY = kP*Py + kD*Dy
        
        Pz = EError.z
        Dz = EError.z - self.lastErrorZ
        SolutionZ = kP*Pz + kD*Dz
        
        #Apply the computed torque to the capsule body
        #self.w.active[joint.child].body.addTorque( (-SolutionX, -SolutionY, -SolutionZ) )
        joint.motor.setParam( luya.ParamVel,   SolutionX )
        joint.motor.setParam( luya.ParamVel2, SolutionY )
        joint.motor.setParam( luya.ParamVel3, SolutionZ )
        
        #Record the last Error
        self.lastErrorX = EError.x
        self.lastErrorY = EError.y
        self.lastErrorZ =EError.z

## Initialize our PD Controller
def init(g):
    g.w = luya.World( ground=True )
    g.jointPIDs = []
    
    for i in g.w.joints.values():
        g.jointPIDs.append( jointPID( g.w, i, 1, 0, 1 ) )
    

## Run Our PD Controller
def run(g):
    for pid in g.jointPIDs:
        pid()
        
    g.w.step( 0.05 )

def end(g):
    pass
