#Modules needed by the current module. These can be thought of as includes and libraries in one thing.
from maya.api import MVector,MQuaternion
from math import acos,degrees,radians
from mayaUtils import *
from luyaUtils import *
import luyaODE as luya

halfPi = luya.pi / 2

def calcTorque(torque,upV,thiV,motor):
    upV.normalize()
    thiV.normalize()
    
    lAxis = (upV ^ thiV)
    lA = upV ^ lAxis
    lError = thiV.angle(lA) - halfPi
    lEuler = MQuaternion(torque(lError),lAxis)
    
    strength = -1
    #In the below example ODE velocities can only be set as Euler rotations.
    motor.setParam( luya.ParamVel,   lEuler.x*strength )
    motor.setParam( luya.ParamVel2, lEuler.y*strength )
    motor.setParam( luya.ParamVel3, lEuler.z*strength )

def equation( balance ):
    return cos(balance*(halfPi))

def init( g ):
    #w.dynamics.shin.body.addForce((10000,0,0))
    g.torque = PID(10,0,0)
    g.bSwitch = True
    g.w=luya.World( ground=True )
    
def run( g ):
    base = Translate("b")
    knee = Translate("knee") - base
    shin = Translate("shin") - base
    thi = Translate("thi") - knee
    
    cogX = MVector( - knee.x, 0, - knee.z ) * 2
    balance = cogX.length() / shin.length()
    
    print "Frame %s : %s" % ( g.time, balance)
    
    if balance < 1:
        cogY = MVector( 0, sqrt( pow(shin.length(),2) - pow(cogX.length(),2) ), 0 )
        cog = cogY + cogX
        oX = cogX.normal()
        oY = MVector(0,1,0)
        
        offset = balance
        setPoint = (cog.normal()*offset) + (oY*(1-offset))
        
        Translate("cog",cog+knee+base) ; setKeyframe("cog")
        Translate("setPoint",setPoint+knee+base) ; setKeyframe("setPoint")
        Translate("x",oX+knee+base) ; setKeyframe("x")
        Translate("y",oY+knee+base) ; setKeyframe("y")
        
        calcTorque(g.torque,setPoint,thi,g.w.dynamics.knee.motor)
    else:
        if g.bSwitch:
            print "Bale out: %s" % g.time
            g.bSwitch = False
            
    g.w.step(0.01)
    
def end(g):
    pass


