## Given a Current Rotation and and Reference position Compute an Angular Velocity.

import Fuzzy; reload(Fuzzy)

from mayaUtils import Translate

#import fuzzyRules
#reload(fuzzyRules)
from fuzzyRules import interpret

from math import degrees,radians
from maya.api import MQuaternion,MVector

## Given a Current Rotation and and Reference position Compute an Angular Velocity.
#
# This jointTracker uses fuzzy logic to compute the angular velocity needed to get
# a joint ( represented by a vector ) to a predestined rotation ( also a vector ).
# You may need a motor to bring the joint to the specified Angular Velocity.
#
# The Following example shows an inverse pendulum controller using jointTracker.
# @code
#    motor = myPhysicsMotor()
#    joint = myPhysicsJoint()
#    tracker = jointTracker()
#    for i in range(0,24):
#        Angle = joint.AngleFromUpVector()
#        AngularVelocity = jointTracker( Angle )
#        motor.setAngularVelocity( AngularVelocity )
# @endcode
class jointTracker:
    def __init__( self, strength=180 ):
        """
            Define the Angle Input Variable.
        """ 
        scale  = 90
        u = Stepper(-scale*3, scale*3,[ -3*scale, -scale, scale, 3*scale ])
        self.Angle = InputVariable (u.lower,u.upper,(
            ("Low" , Linear(u(0),u(1)), Linear(u(1),u(2))),
            ("High", Linear(u(1),u(2)), Linear(u(2),u(3)))
        ))
        
        u = Stepper(-strength*3, strength*3,[ -3*strength, -strength, strength, 3*strength ])
        self.AngularSpeed = OutputVariable (u.lower,u.upper,(
            ("Low" , Linear(u(0),u(1)), Linear(u(1),u(2))),
            ("High", Linear(u(1),u(2)), Linear(u(2),u(3)))
        ))
        
        self.ruleSet = interpret(\
        """
            if Angle is High : AngularSpeed = High
            if Angle is Low : AngularSpeed = Low
        """
        )
        
    ## Do the computation
    # @param Current is the Current state. A normalised vector representing the current direction
    # @param Reference is where you want to be. A normalised vector representing the destination direction.
    #       If Reference is not provided then it is set to (0,1,0) the upvector. This will compute the solution to the inverse pendulum.
    # @param log File for debugging purposes.
    def __call__( self, Current, Reference=MVector(0,1,0), log=None ):
        Angle = self.Angle ; AngularSpeed = self.AngularSpeed
        
        """Normalize the two input vectors"""
        Reference = Reference.normal()
        Current = Current.normal()
        
        """
            Compute the rotation axis.
            The rotation axis flips when Current changes which side of Reference it is on.
            This is what we use to determin the Direction part of the Angular Velocity.
        """
        Axis = ( Reference ^ Current ).normal()
        
        """
            Work out the current Error value.
            This is the angle between Current and Reference
        """
        Error = degrees( Current.angle( Reference ) )
    
        """Fuzzify the Error value"""
        Angle.fuzz(Error)
        
        """
            Prepare the output variable for the computation of the Fuzzy Ruleset.
            Any accumulated membership values are cleared out.
        """
        AngularSpeed.begin()
        
        """
            Execute the fuzzy rules.
            This computes an accumulative membership value for each set in the output variable AngularSpeed.
        """
        exec(self.ruleSet)
        
        """
            Defuzz the Angular velocity.
            Get a "crisp" value from the AngularSpeed Linguistic Variable.
        """
        Solution = AngularSpeed.defuzz()
        
        """
            For debugging purposes write to a log if a log file was provided as
            and arguement to the call function.
        """
        if log != None:
            log.write( "Error: %s Solution: %s " % (Error,Solution) )
        
        """
            Return thr angular velocity as a Quaternion.
        """
        return MQuaternion( radians( Solution ),Axis)
