## Run the expressions
expressions=["Ragdoll_Character"]	# The current behaviour to run
path="/bacva3/ltitley/Devel/Luya"		#The directory of our python modules
beh_path="/bacva3/ltitley/Devel/Luya/Example_Behaviours/"  # The directory containing our behaviours

## Do not modify anything below this line
import sys
from maya.mel import currentTime, playbackOptions
from maya.api import MGlobal
from time import time as timeSeconds
N = None

## Run a behaviour module function
def runBeh(globo,module,part):
	exec( "module.%s( globo )" % part )

## Retreive start and end times for timeline in maya.
#
#This script is sourced, so we have to be carefull not to declare any global variables
#which might confuse the debugging process.
#That is why it is wrapped up in a scope function.
#
#
# - Do setup.
# - Set time to start of timeline
# - Run each behaviour's init function
# - Run each behaviour's run function
# - Move on to next frame
def main(beh_names):
	
	### Set up the system paths for the directories which contain module
	sys.path.append(path)
	sys.path.append(path+"/fuzzy_logic/")
	sys.path.append(beh_path)
	
	### import the standard luyaUtils module used by everything
	import luyaUtils
	globo = luyaUtils.globo()
	
	### Get start and end frames"""
	start = int(playbackOptions(q=N,min=N))
	end = int(playbackOptions(q=N,max=N))
	
	### Some usefull information to pass to the behaviours
	globo.start = start
	globo.end = end
	globo.time = start
	
	currentTime(start,e=N)
	
	### Import and run the init function for the behaviour
	beh = []
	for i in beh_names:
		k = __import__(i,globals(),locals(),[])
		reload(k)
		beh.append(k)
		runBeh(globo,k,"init")
	
	globo.startSeconds = timeSeconds()
	### Run the actual run function
	for time in range(start,end+1):
		currentTime(time,e=N)
		globo.time = time
		for i in beh:
			runBeh(globo,i,"run")

	globo.endSeconds = timeSeconds()
	MGlobal.displayInfo("It took %s seconds to bake simulation.\n" % (globo.endSeconds - globo.startSeconds) )
	
	### Run the end function
	for i in beh:
		runBeh(globo,i,"end")
	
	currentTime(start,e=N)

main(expressions)
