import maya.mel as mel 
import mayaUtils
reload(mayaUtils)
from maya.api import MGlobal as M
from maya.api import MEulerRotation
from maya.api import MQuaternion
from maya.api import MVector
from maya.api import MSelectionList
from math import *
import ode
N = None

"""
	Thanks to Paul Kinnane on the pyODE forum/ message board for the information on the Param ... 3 arguments.
	Genius.
"""
ParamLoStop = ode.ParamLoStop
ParamHiStop = ode.ParamHiStop
ParamVel = ode.ParamVel
ParamFMax = ode.ParamFMax

ParamLoStop2 = ode.ParamLoStop2
ParamHiStop2 = ode.ParamHiStop2
ParamVel2 = ode.ParamVel2
ParamFMax2 = ode.ParamFMax2

ParamLoStop3 = 512+0
ParamHiStop3 = 512+1
ParamVel3 = 512+2
ParamFMax3 = 512+3

class atom:
	pass

def near_callback(w,geom1,geom2):
	
	for i in w.collisionSet.values():
		if i.has_key(geom1) and i.has_key(geom2):
			return
	
	contacts = ode.collide(geom1,geom2)
	
	for i in contacts:
		j = ode.ContactJoint(w.w,w.contacts,i)
		i.setBounce(0.02)
		i.setMu(1000000)
		j.attach( geom1.getBody(), geom2.getBody() )

def isRigidTransform(name):
	"""Given the name of a transform, this returns true if the transform contains """
	shapes = mel.listRelatives(name,f=N,c=N,s=N)
	if len(shapes) > 0:
		nodeType = mel.nodeType(shapes[0])
		if nodeType == "luCapsule":
			return True
		elif nodeType == "luBox":
			return True
	return False

def hasDynamicParent(name):
	#Get the parent
	parent = mel.listRelatives(name,f=N,p=N)
	for i in parent:
		if isRigidTransform(i):
			return i
	for i in parent:
		j = hasDynamicParent(i)
		if j != None:
			return j
	return None

def hasDynamicChild(name):
	#Get the parent
	child = mel.listRelatives(name,f=N,c=N)
	for i in child:
		if isRigidTransform(i):
			return i
	for i in child:
		j = hasDynamicChild(i)
		if j != None:
			return j
	return None

def isDynamicJoint(name):
	"""Given the name of a joint, return true if it is dynamic, also return the name of the parent and child.
		If a joint has no child then it is not dynamic and (False,None,None) is returned.
	"""
	#Get the parent
	parent = hasDynamicParent(name)
	#Get the children
	child = hasDynamicChild(name)
	
	#Check the joint has a child rigid transform, if it does it is a dynamic joint
	if child != None:
		return (True,parent,child)
	else:
		return (False,None,None)

class DynamicBody:
	def __init__(self,w,shapeName,attributes=[]):
		## The Maya Object
		transformName = mel.listRelatives(shapeName,p=N,f=N)[0]
		
		#Find out if it is a root rigid body and if so, add it to a list
		if len( mel.listRelatives(transformName,p=N,f=N) ) == 0:
			w.roots.append(transformName)
			
		#Attributes
		#Transform
		position = mel.xform(transformName,q=N,ws=N,t=N)
		rotation = mel.xform(transformName,q=N,ws=N,ro=N)

		#E = MEulerRotation( radians(rotation[0]), radians(rotation[1]), radians(rotation[2]) )
		#Q = E.asQuaternion()

		#Material
		density = mel.getAttr("%s.mass" % shapeName)

		#Shape specific attributes
		self.attributes = atom()
		self.attrs = self.attributes
		
		for i in attributes:
			self.attributes.__dict__[i] = mel.getAttr("%s.%s" % (shapeName,i))
		
		## The Mass and Physics Body
		if density != 0:
			body = ode.Body(w.w)
			#body.setPosition( position )
			#body.setQuaternion((Q.w,Q.x,Q.y,Q.z))
			self.body = body
			
		## Packaging it up
		self.density = density
		self.name = transformName
		self.transform =[	1,0,0,0,
							0,1,0,0,
							0,0,1,0,
							0,0,0,1
						]
		#self.position = position
		#self.quaternion = (Q.x,Q.y,Q.z,Q.w)
		
		## Adding it to the world class
		if density == 0:
			w.passive[transformName] = self
		else:
			w.active[transformName] = self
		w.dynamics.__dict__[transformName.replace("|","_")[1:]] = self
		
	def getTransformFromODE(self):
		if self.isActive() :
			P = self.body.getPosition()
			R = self.body.getRotation()
		else:
			P = self.collision.getPosition()
			R = self.collision.getRotation()
			
		transform = (	R[0], R[3], R[6], 0.,
						R[1], R[4], R[7], 0.,
						R[2], R[5], R[8], 0.,
						P[0], P[1], P[2], 1.0
					)

		return transform
	
	def setTransformToODE(self,T):
		rotation = (	T[0], T[4], T[8],
						T[1], T[5], T[9],
						T[2], T[6], T[10]
					)
		position = ( T[12], T[13], T[14] )
		if self.isActive() :
			self.body.setPosition(position)
			self.body.setRotation(rotation)
		else:
			self.collision.setPosition(position)
			self.collision.setRotation(rotation)
			
	def setTransformToMaya(self,T):
		try:
			mel.xform(self.name,ws=N,a=N,matrix=T)
		except RuntimeError:
			M.displayInfo("Runtime error in Luya. Caused by invalid matrix information. Luya is assuming your rig has exploded.")

	def setTransformFromODE(self):
		transformO = self.getTransformFromODE()
		self.setTransformToMaya(transformO)
	
	def setTransformFromMaya(self):
		transformM = mel.xform(self.name,q=N,ws=N,a=N,matrix=N)
		self.setTransformToODE(transformM)
		
	def isActive(self):
		if self.density == 0:
			return False
		else:
			return True

class World:
	def __init__(self, gravity=(0,-9.8,0), ground=True ):
		self.w = ode.World()
		self.world = self.w
		self.space = ode.Space()
		self.contacts = ode.JointGroup()
		self.w.setGravity( gravity )
		
		self.active = {} #Active Rigid Bodies
		self.passive = {} #Active Rigid Bodies
		self.roots = [] #Root Rigid Bodies
		self.joints = {} #Joints
		self.collisionSet = {} # Collision Sets
		self.dynamics = atom() #Joints and Rigid Bodies via names
		
		if ground : self.ground = ode.GeomPlane(self.space,normal=(0,1,0))
		
		#Parse the maya scene and find all the rigid bodies and joints to be simulated
		self.parseScene()
		
		mayaUtils.clearKeyframes(self.passive.keys())
		mayaUtils.clearKeyframes(self.active.keys())
		mayaUtils.clearKeyframes(self.joints.keys())
		
		self.cleanup()
		
	def end(self):
		pass
		
	def parseScene( self ):
		"""
			Parses the maya scene to find all rigid bodies and joints.
		"""
		#Looking for capsules
		for shapeName in mel.ls(type="luCapsule",l=N):
			## The Maya Object
			a = DynamicBody(self,shapeName,["radius","length"])
			
			#Shape Attributes
			radius = a.attrs.radius
			length = a.attrs.length

			## The Collision Object
			a.collision = ode.GeomCCylinder(self.space,radius=radius,length=length)
			
			## The Mass and Physics Body
			if a.isActive() :
				a.mass = ode.Mass()
				a.mass.setCappedCylinder(a.density,3,radius,length)
				a.body.setMass(a.mass)
				a.collision.setBody(a.body)
			
			a.setTransformFromMaya()
			
		#Looking for boxes
		for shapeName in mel.ls(type="luBox",l=N):
			
			## The Maya Object
			a = DynamicBody(self,shapeName,["height","width","depth"])
			
			#Shape Specifics
			height = 2*a.attrs.height
			width = 2*a.attrs.width
			depth = 2*a.attrs.depth
			
			## The Collision Object
			a.collision = ode.GeomBox(self.space,lengths=(height,width,depth))
			
			## The Mass and Physics Body
			if a.isActive() :
				a.mass = ode.Mass()
				a.mass.setBox(a.density,height,width,depth)
				a.body.setMass(a.mass)
				a.collision.setBody(a.body)
			
			a.setTransformFromMaya()
		
		#Looking for joints which have 1 parent and 1 child capsule
		#Select all joints
		for i in mel.ls(type="joint",l=N):
			#Check that it is dynamic, and if it is, get its parent and child
			(isDynamic,parent,child) = isDynamicJoint(i)
			
			if isDynamic :
				#The two body objects below are ode objects
				#Get the parent body, if the joint has no parent, then attach it to the world.
				parentBody = ode.environment
				if parent != None:
					if self.active.has_key(parent):
						parentBody = self.active[parent].body
				
				#Get the child body
				childBody = self.active[child].body
				
				#Get the postion of the joint
				position = mel.xform(i,q=N,ws=N,t=N)
				joint = ode.BallJoint( self.w )
				joint.attach( parentBody, childBody )
				joint.setAnchor( position )
				
				#Packaging it up
				a = atom()
				a.joint = joint
				a.name = i
				
				#Adding it to the world class
				self.joints[i] = a
				if parent == None:
					self.roots.append(i)
				self.dynamics.__dict__[i.replace("|","_")[1:]] = a
		
		#Remove any keyframes which might already exist
		self.cleanup()
	
	def cleanup(self):
		for i in self.active.keys():
			attrs=[	"translateX","translateY","translateZ",
					"rotateX","rotateY","rotateZ",
					"scaleX","scaleY","scaleZ",
					"visibility"
			]
			for j in attrs:
				attrname = "%s_%s" % (i,j)
				if mel.objExists(attrname):
					mel.delete(attrname)

	def m_subStep(self,name):
		"""This is an internal function, don't use it.
			It recursively travels down the hierarchy of a rig and keys the relevent transforms.
		"""
		if len(name) > 0:
			objects = self.active
			j = self.joints
			
			#Is it a rigid body
			if self.active.has_key(name) :
				self.active[name].setTransformFromODE()
				mel.setKeyframe(name)
				
			#Is it a joint
			if j.has_key(name):
				pos = self.joints[name].joint.getAnchor()
				mel.xform(name,ws=N,t=pos)
				mel.setKeyframe(name)
			
			children = mel.listRelatives(name,f=N,c=N,type=["transform","joint"])
			for i in children:
				self.m_subStep(i)
		
	def step(self,timestep=1):
		# Go through all the passive rigid bodies and update there transforms based on what they're given
		# from Maya.
		for i in self.passive.values():
			i.setTransformFromMaya()
				
		for i in self.roots:
			self.m_subStep(i)
			
		self.space.collide( self, near_callback )
		self.w.step(timestep)
		self.contacts.empty()
		