from maya.mel import *
from maya.api import MVector,MQuaternion,MGlobal,MMatrix

class atom:
    pass

N = None

class MayaAttr:
	"""Wraps around the maya attributes. Rather than getting and setting attributes.
		We can change the values of the attributes directly.
	"""
	def __init__(self,name,time=None):
		self.name = name
		if time != None:
			self.time = time
		else:
			self.time = currentTime(q=N)

	def __getitem__(self,i):
		return self(i)
	
	def __call__(self,value=None):
		if objExists(self.name):
			if value == None:
					return getAttr(self.name,time=self.time)
			else :
				assert (self.time == currentTime(q=N),"This attribute is read only because it is not set to the current time. You cannot change attributes at a different time than the current.")
				setAttr(self.name,value)
		return 0
			
	def __add__(self,value):
		return self() + value
	
	def __sub__(self,value):
		return self() - value
	
	def __mul__(self,value):
		return self() * value
	
	def __div__(self,value):
		return self() / value
	
	def __iadd__(self,value):
		self( self + value )
		
	def __isub__(self,value):
		self( self - value )
	
	def __imul__(self,value):
		self( self * value )
		
	def __idiv__(self,value):
		self( self / value )

def doc(c):
	print c.__doc__

def m4tox( m4 ):
    m4 = (    m4[0][0], m4[0][1], m4[0][2], m4[0][3],
               m4[1][0], m4[1][1], m4[1][2], m4[1][3],
               m4[2][0], m4[2][1], m4[2][2], m4[2][3],
               m4[3][0], m4[3][1], m4[3][2], m4[3][3]
           )
    return m4

def m4toR( m4 ):
    m4 = MMatrix([    [m4[0][0], m4[0][1], m4[0][2], m4[0][3]],
                       [m4[1][0], m4[1][1], m4[1][2], m4[1][3]],
                       [m4[2][0], m4[2][1], m4[2][2], m4[2][3]],
                       [0, 0, 0, 1.0]
           ])
    return m4

def xto4( R ):
    m4 = MMatrix([    [R[0],    R[1], R[2],    R[3]],
                    [R[4],    R[5], R[6],    R[7]],
                    [R[8],    R[9], R[10],R[11]],
                    [R[12],    R[13],R[14],R[15]]
                ])
    return m4

def pto4( p ):
    m4 = MMatrix([    [1.0,    0.0, 0.0,    0.0],
                    [0.0,    1.0, 0.0,    0.0],
                    [0.0,    0.0, 1.0,    0.0],
                    [p[0], p[1], p[2],    1.0]
                ])
    return m4

def m3to4( R, p ):
    m4 = MMatrix([    [R[0],    R[3], R[6],    0.0],
                    [R[1],    R[4], R[7],    0.0],
                    [R[2],    R[5], R[8],    0.0],
                    [p[0],    p[1], p[2],    1.0]
                ])
    return m4

def Translate(name,t=None):
	if t == None:
		return MVector(xform(name,ws=N,q=N,t=N))
	else:
		xform(name,ws=N,t=(t[0],t[1],t[2]))
        
def Transform(name,t=None):
    if t == None:
        return xto4( xform( name,ws=N,q=N,matrix=N ) )
    else:
        xform( name,ws=N,matrix= m4tox(t) )

def returnByName(name,type="transform"):
    "Returns an array of all the node of type <transform> which contain the name <name>"
    objs = ls(type=type)
    ret_array = []
    for i in objs:
        if i.find(name) != -1:
           ret_array.append(i)
    return ret_array

def linear(t, fmin=0.0, fmax=1.0, nmin=0.0, nmax=1.0):
	"""Maps a linear interpolation from fmin-fmax to nmin-nmax"""
	fmin = float(fmin)
	fmax = float(fmax)
	nmin = float(nmin)
	nmax = float(nmax)
	t = (t-nmin)/(nmax-nmin)
	return (fmin*(1-t)) + (fmax*t)

def copyAttrs(srcName,dstName,srcAttrs,dstAttrs):
	assert len(srcAttrs) == len(dstAttrs) , "The attribute arrays given are not of the same length"
	for i in range(0,len(srcAttrs)):
		attrValue = getAttr( "%s.%s" % (srcName,srcAttrs[i]) )
		setAttr( "%s.%s" % (dstName,dstAttrs[i]), attrValue )

def selectHierarchyByType(i,jointList,type="joint"):
	try:
		if nodeType(i) == type:
			jointList.append(i)
		children = listRelatives(i,c=N)
		for i in children:
			selectHierarchyByType(i,jointList,type=type)
	except RuntimeError :
		MGlobal.displayInfo("Use Unique Names in your scene! It saves a lot of trouble")

def getVector3(name,attrname,time=None):
	if time == None:
		x = getAttr("%s.%sX" % (name,attrname))
		y = getAttr("%s.%sY" % (name,attrname))
		z = getAttr("%s.%sZ" % (name,attrname))
	else:
		x = getAttr("%s.%sX" % (name,attrname),time=time)
		y = getAttr("%s.%sY" % (name,attrname),time=time)
		z = getAttr("%s.%sZ" % (name,attrname),time=time)
	return MVector(x,y,z)

def setVector3(name,attrname,attr):
	setAttr("%s.%sX" % (name,attrname), attr[0])
	setAttr("%s.%sY" % (name,attrname), attr[1])
	setAttr("%s.%sZ" % (name,attrname), attr[2])

def copyAttrs(srcName,dstName,srcAttrs,dstAttrs):
	assert len(srcAttrs) == len(dstAttrs) , "The attribute arrays given are not of the same length"
	for i in range(0,len(srcAttrs)):
		attrValue = getAttr( "%s.%s" % (srcName,srcAttrs[i]) )
		setAttr( "%s.%s" % (dstName,dstAttrs[i]), attrValue )

def promptValue(prompt):
	"""Produce a Dialog box as input to a utility function"""
	result = promptDialog(
		title=prompt,
		message=prompt,
		button=["OK","Cancel"],
		defaultButton="OK",
		cancelButton="Cancel",
		dismissString="Cancel"
	)
	return promptDialog(q=N,text=result)

def fullname( name ):
    sl = ls(sl=N)
    select(cl=N)
    select(name)
    fullname = ls(sl=N,l=N)[0]
    select(cl=N)
    if len(sl) > 0:
        select(sl,add=N)
    return fullname


##    Clear the Attributes of the tranforms keyed by luya.
#       The Animation curve nodes are deleted themselves rather than just breaking the connections.
deleteList = [ "translateX",
               "translateY",
               "translateZ",
               "rotateX",
               "rotateY",
               "rotateZ",
               "scaleX",
               "scaleY",
               "scaleZ",
               "visibility"
              ]

def clearObject( i ):
    for k in deleteList:
        MGlobal.executeCommand("CBdeleteConnection %s.%s" % (i,k) )

if __name__ == "__main__":
    clearJoints()
    
    