"""
 PID Controller for balancing the inverse pendulum.
 The maya scene must have the following locators:
     AimTip
     AimBase
 The maya scene must have the following joint:
     knee
     thi
"""

def makeMotor(w,knee):
    parent = knee.getBody(0)
    child = knee.getBody(1)
    
    cap = luya.ode.AMotor(w.w)
    cap.attach(parent, child)
    
    cap.setMode(luya.ode.AMotorEuler)
    
    cap.setAxis( 0, 1, (1,0,0 ))
    cap.setAxis( 2, 2, (0,0,1 ))
    
    cap.setAngle(0,0)
    cap.setAngle(1,0)
    cap.setAngle(2,0)
    
    mstop = halfPi
    strength = 40000
    
    cap.setParam(luya.ParamFMax,strength)
    cap.setParam(luya.ParamLoStop,-mstop)
    cap.setParam(luya.ParamHiStop,mstop)
    
    cap.setParam(luya.ParamFMax2,strength)
    cap.setParam(luya.ParamLoStop2,-mstop)
    cap.setParam(luya.ParamHiStop2,mstop)
    
    cap.setParam(luya.ParamFMax3,strength)
    cap.setParam(luya.ParamLoStop3,-mstop)
    cap.setParam(luya.ParamHiStop3,mstop)
    
    return cap

def Translate(name,t=None):
    if t == None:
        return MVector(xform(name,ws=N,q=N,t=N))
    else:
        xform(name,ws=N,t=(t[0],t[1],t[2]))

import os
import luya_api as luya
reload(luya)

########Physics Part
halfPi = luya.pi / 2

#Initialize the world
w = luya.World()

w.motors = luya.atom()
w.motors.mPri = makeMotor( w, w.dynamics.base_jointPri.joint )
w.motors.mSec = makeMotor( w, w.dynamics.base_jointPri_shin_jointSec.joint )
w.motors.mThird = makeMotor( w, w.dynamics.base_jointPri_shin_jointSec_thi_jointThird.joint )
