"""
 PID Controller for balancing the inverse pendulum.
 The maya scene must have the following locators:
     AimTip
     AimBase
 The maya scene must have the following joint:
     knee
     thi
"""

for i in range(20):
    w.step(0.02)

from maya.api import MVector,MQuaternion
from math import acos,degrees,radians

class PID:
    """Proportional Integral Differential Controller"""
    def __init__(self,kP,kI,kD):
        self.lastP = 0
        self.i = 0
        self.kP = kP
        self.kI = kI
        self.kD = kD

    def __call__(self,error):
        p = error
        self.i += self.lastP
        d = self.lastP - p
        v = self.kP*p + self.kI*self.i + self.kD*d
        self.lastP = p
        return v

if time == 10:
    cap.addTorques(70000,0,70000)
    torque = PID(1,0,0)
    
else :
    parentV = Translate("knee")
    thiV = (Translate("thi") - parentV).normal()
    upV = Translate("AimTip") - Translate("AimBase")
    
    axis = upV ^ thiV
    a = upV ^ axis
    error = thiV.angle(a) - halfPi
    euler = MQuaternion(torque(error),axis)
    """
        I'm working in quaternions to avoid gimbal lock.
        This also leaves the option open to use other physics engines which don't support
        euler rotations(Bullet).
    """
    
    #In the below example ODE velocities can only be set as Euler rotations.
    cap.setParam( luya.ParamVel, euler.x )
    cap.setParam( luya.ParamVel2, euler.y )
    cap.setParam( luya.ParamVel3, euler.z )

