"""
 PID Controller for balancing the inverse pendulum.
 The maya scene must have the following locators:
     AimTip
     AimBase
 The maya scene must have the following joint:
     knee
     thi
     
 Note to me. Always delete global variables after they have been defined in a run behaviour.
"""

"""
    I'm working in quaternions to avoid gimbal lock.
    This also leaves the option open to use other physics engines which don't support
    euler rotations(Bullet).
"""

for i in range(20):
    w.step(0.02)

from maya.api import MVector,MQuaternion
from math import acos,degrees,radians

def calcTorque(thiV,motor):
    upV = MVector(0,1,0).normal()
    
    lAxis = (upV ^ thiV.normal())
    lA = upV ^ lAxis
    lError = thiV.angle(lA) - halfPi
    lEuler = MQuaternion(torque(lError),lAxis)
    
    print degrees(lError)
    strength = 1
    #In the below example ODE velocities can only be set as Euler rotations.
    motor.setParam( luya.ParamVel,  lEuler.x*strength )
    motor.setParam( luya.ParamVel2, lEuler.y*strength )
    motor.setParam( luya.ParamVel3, lEuler.z*strength )
    
class PID:
    """Proportional Integral Differential Controller"""
    def __init__(self,kP,kI,kD):
        self.lastP = 0
        self.i = 0
        self.kP = kP
        self.kI = kI
        self.kD = kD

    def __call__(self,error):
        p = error
        self.i += self.lastP
        d = self.lastP - p
        v = self.kP*p + self.kI*self.i + self.kD*d
        self.lastP = p
        return v

if time == 40:
    w.dynamics.base_jointPri_shin_jointSec_thi_jointThird_hip.body.addForce((100000,0,0))

if time == 1:
    #w.motors.mSec.addTorques(100000,0,0)
    torque = PID(1,0,0)
    #upperTorque = PID(1,0,0)
    
else :
    base = Translate("shin") - Translate("jointPri")
    tip = Translate("thi") - Translate("jointSec")
    tiptip = Translate("hip") - Translate("jointThird")
    
    print "Base"
    calcTorque(base,w.motors.mPri)
    print "Tip"
    calcTorque(tip,w.motors.mSec)
    print "TipTip"
    calcTorque(tiptip,w.motors.mThird)

