"""
 PID Controller for balancing the inverse pendulum.
 The maya scene must have the following locators:
     AimTip
     AimBase
 The maya scene must have the following joint:
     knee
     thi
     
 Note to me. Always delete global variables after they have been defined in a run behaviour.
"""

"""
    I'm working in quaternions to avoid gimbal lock.
    This also leaves the option open to use other physics engines which don't support
    euler rotations(Bullet).
"""

for i in range(1):
    w.step(0.06)

from maya.api import MVector,MQuaternion
from math import acos,degrees,radians

def calcTorque(torque,upV,thiV,motor):
    upV.normalize()
    thiV.normalize()
    
    lAxis = (upV ^ thiV)
    lA = upV ^ lAxis
    lError = thiV.angle(lA) - halfPi
    lEuler = MQuaternion(torque(lError),lAxis)
    
    strength = 1
    #In the below example ODE velocities can only be set as Euler rotations.
    motor.setParam( luya.ParamVel,  lEuler.x*strength )
    motor.setParam( luya.ParamVel2, lEuler.y*strength )
    
    motor.setParam( luya.ParamVel3, lEuler.z*strength )
    
class PID:
    """Proportional Integral Differential Controller"""
    def __init__(self,kP,kI,kD):
        self.lastP = 0
        self.i = 0
        self.kP = kP
        self.kI = kI
        self.kD = kD

    def __call__(self,error):
        p = error
        self.i += self.lastP
        d = self.lastP - p
        v = self.kP*p + self.kI*self.i + self.kD*d
        self.lastP = p
        return v

def equation( balance ):
    return cos(balance*(halfPi))

if time == 1:
    #w.dynamics.shin.body.addForce((10000,0,0))
    torque = PID(20,0,-1)
    bSwitch = True
    print "\n\n\n"
    print "########################### Starting ##################################"

else :
    base = Translate("b")
    knee = Translate("knee") - base
    shin = Translate("shin") - base
    thi = Translate("thi") - k
    
    cogX = MVector( - knee.x, 0, - knee.z ) * 2
    balance = cogX.length() / shin.length()
    
    print "Frame %s : %s" % ( time, balance)
    
    if balance < 1:
        cogY = MVector( 0, sqrt( pow(shin.length(),2) - pow(cogX.length(),2) ), 0 )
        cog = cogY + cogX
        oX = cogX.normal()
        oY = MVector(0,1,0)
        
        offset = balance
        setPoint = (cog.normal()*offset) + (oY*(1-offset))
        
        Translate("cog",cog+knee+base) ; setKeyframe("cog")
        Translate("setPoint",setPoint+knee+base) ; setKeyframe("setPoint")
        Translate("x",oX+knee+base) ; setKeyframe("x")
        Translate("y",oY+knee+base) ; setKeyframe("y")
        
        calcTorque(torque,setPoint,thi,w.motors.mKnee)
    else:
        if bSwitch:
            print "Bale out: %s" % time
            bSwitch = False

