require("custardCream")

-----------------------------------------------------------------------------------------------
-- point(ray) , plane intersection 
-----------------------------------------------------------------------------------------------
function intersect(rOrigin, rNormal, pOrigin, pNormal)
    local D = pOrigin:dot(pNormal);

    local numer = pNormal:dot(rOrigin) + D;
    local denom = pNormal:dot(rNormal);
    return 0-(numer / denom);
end

-----------------------------------------------------------------------------------------------
-- main stuff:
-----------------------------------------------------------------------------------------------

--- settings ---

local planePoint = Vector(0,10,0)
local planeNormal = Vector(0.2,1,-0.4)
local radius = 2
local particleAmount = 500
local particleRate = 10

local startVec = Vector(-10,20,-10)

----------------

local p = Particles(particleAmount)

local points = p:getPosition()
local widths = p:getWidths()
local colours = p:getColours()
local velocity = p:getVelocity()
local mass = p:getMass()

for i=0, points:size()-1 do
	widths:setElement(i, Float(radius))
	points:setElement(i, startVec)
end

camera:transform(Translate(0, 0, 500))
camera:setRes(720, 576)

local plane = Plane(5,5)
plane:transform(Scale(200,200,200)*Rotate(3.1415/2,0,0)*Aim(Vector(0,1,0),planeNormal)*Translate(0,-10,0))
Render(plane)

local c1 = p
local c2 = p
local b1
local b2
local b = nil
for frame=1, 100 do
	
	local at = Vector(0,0- 9.80665/25 ,0) 
	for i=0, points:size()-1 do
		if i > frame*particleRate then	
			points:setElement(i, startVec)
		else
			velocity:setElement(i, velocity:getElement(i)+at)
		end
	end
	
	startVec = startVec+Vector(Rand(0,1),0,Rand(0,1))
	
	-- point point collision --
	local count = 0
	for i=0, (points:size()-1) *0.5 do
		for j=math.floor((points:size()-1) *0.5), points:size()-1 do
			if (points:getElement(i)-points:getElement(j)):length() <= radius*2 then
	
				local c = Vector(0.5)
				
				local V1 = velocity:getElement(i)
				local V2 = velocity:getElement(j)
				
				local M1 = Vector(mass:getElement(i):getData())
				local M2 = Vector(mass:getElement(j):getData())
				
				local VF1 = ((c+Vector(1))*M2*V2 + V1*(M1-c*M2))/(M1+M2)
				local VF2 = ((c+Vector(1))*M1*V1 + V2*(M2-c*M1))/(M2+M1)
				
				velocity:setElement(i, VF1)
				velocity:setElement(j, VF2)
				
				count = count + 1
			--	print(i.." "..j)
			end
		end	
	end

	-- floor collision --
	for i=0, points:size()-1 do		
		local t =  math.abs(
			intersect(points:getElement(i), velocity:getElement(i), planePoint, planeNormal))
			
		if t <= 1.1  then		
			print("Frame: "..frame.." | "..i.." | "..t)
			
			velocity:setElement(i, 
				Vector(0)-
				((Vector(velocity:getElement(i):dot(planeNormal)*2)*planeNormal)-
					velocity:getElement(i)):scale(
						Vector(Rand(0,0.5),Rand(0,0.5),Rand(0,0.5))))			
		end
	end
	
	--------------------

	print("   Collilsions: "..count)

	if frame ~= 1 then	
		unRender(b)
	end
	
	if frame ~= 1 then	
		b1 = b2
	end
	
	local curves = Curves()	
	local k=0
	for i=0, (points:size()-1)/particleRate do
		local c = AttributeVector(10)
		for j=0, 9 do		
			c:setElement(j, points:getElement(k))
			k=k+1
		end
		curves:addCurve(c)
	end

	
	p2 = Particles(0,particleAmount)
	for i=0, (curves:getAmount()-1) do
		local counter = 0
		local totalBs = 300
		local ca = curves:CurveLength(i) *3
		if ca ~= 0 then			
			for j=0, math.floor(ca) do
				local w = (1-((1/ca)*j))^2
				p2:addPoint(curves:getPointAt(i,(1/ca)*j), w)
				counter=counter+1
			end
			for j=0, math.floor(totalBs-ca) do
				p2:addPoint(curves:getPoint(i,0))
				counter=counter+1
			end
		--	print(math.floor(ca)+math.floor(50-ca))
		else
			for j=0, math.floor(totalBs) do
				p2:addPoint(curves:getPoint(i,0))
				counter=counter+1
			end			
		end
		if counter >= totalBs+2 then
			print(i..": "..counter.." > "..totalBs)
		end
	end
	
	c1:nextFrame()	
	
	bo = Blobbies(p2)
	b2 = bo

	if frame ~= 1 then	
		b = Blurred(b2, b1)
		Render(b)
	end
	
	camera:outputTIF("renders/particles8."..pad(frame)..".tif")
	output("RIB", "ribs/particles8."..pad(frame)..".rib")	
	
end

