require("custardCream")

------------------------------------------------------------------------------------------------------------------

function randomPointOnQuad(mesh, element)

	if mesh.points:size() == 4 then 
		local xR = Vector(Rand())
		local yR = Vector(Rand())

		local topP = (xR*mesh.points:getElement(0)) + ((Vector(1)-xR)*mesh.points:getElement(1))
		local bottomP = (xR*mesh.points:getElement(3)) + ((Vector(1)-xR)*mesh.points:getElement(2))

		local finalP = (yR*topP) + ((Vector(1)-yR)*bottomP)
		
		local topN = (xR*mesh.normals:getElement(0)) + ((Vector(1)-xR)*mesh.normals:getElement(1))
		local bottomN = (xR*mesh.normals:getElement(3)) + ((Vector(1)-xR)*mesh.normals:getElement(2))

		local finalN = (yR*topN) + ((Vector(1)-yR)*bottomN)	

		local out = {}
		out.point = finalP
		out.normal = finalN

		return out
	else
--		print("Fur: polygon is non quad: "..mesh.points:size())
	end
end

function makeFurOnPoly(meshIn, element, curves, density)
	
	local mesh = {}
	mesh.points = meshIn:getPolygon(element) 
	mesh.normals = meshIn:getPolygonNormals(element)
	
	for i=1, density do
		local length = Vector(0-Rand(0.1, 0.5))
		local blade = AttributeVector(0,4)
		local point = randomPointOnQuad(mesh, element)
		if point then
			blade:setElement(0, point.point)
			blade:setElement(1, point.point)
			blade:setElement(2, point.point - point.normal * length )
			blade:setElement(3, point.point - point.normal * length )
			curves:addCurve(blade)
		end
	end
	
end

function makeFur(meshIn, density, width)
	
	if density == nil then
		density = 5
	end
	
	if width == nil then 
		width = 0.01
	end
		
	local curves = Curves();

	for i=0, (meshIn:getNPolys()):size()-1 do
		makeFurOnPoly(meshIn, i, curves, density)
	end

	print("Fur Done. "..((meshIn:getNPolys()):size() * density) )

	curves:setWidth(Float(width))
	
	return curves
end

------------------------------------------------------------------------------------------------------------------
--[[
Render(Transform(Rotate(45,0,0) * Translate(0,0,5)))

local plane = Plane(60, 60)
print("Created Plane.")
plane:generateNormalsDirty();	
print("Generate Normals.")
local points = plane:getPoints()
print("Got Points: \""..points:size().."\".")
for i=0, points:size()-1 do
	point = points:getElement(i);
	
	noise = (
			voronoi_F1(point:getX()*20.0, point:getY()*20.0 ,0) *0.1 
			+ voronoi_F1(point:getX()*10.0, point:getY()*10.0 ,0) *0.05
			+ newPerlin(point:getX()*50, point:getY()*50, 0) * 0.02
			)
			* newPerlin(point:getX()*5, point:getY()*5, 0) 
						
	points:setElement(i,  Vector(  point:getX(), point:getY(), noise ))
end
print("Added Noise.")
plane:generateNormalsDirty();	
print("Generate Normals.")


------------------------------------------------------------------------------------------------------------------

local curves = Curves();

for i=0, (plane:getNPolys()):size()-1 do
	makeFurOnPoly(plane, i, curves)
end

print("Fur Done. "..((plane:getNPolys()):size() * 5) )

curves:setWidth(Float(0.001))
Render(curves)

Render(plane)

outputRIB("fur.rib")
--outputGL()
]]--
