require("custardCream")

------------------------------------------------------------------------------------------------------------------

function randomPointOnQuad(mesh, element)

	local xR = Vector(Rand())
	local yR = Vector(Rand())

	local topP = (xR*mesh.points:getElement(0)) + ((Vector(1)-xR)*mesh.points:getElement(1))
	local bottomP = (xR*mesh.points:getElement(3)) + ((Vector(1)-xR)*mesh.points:getElement(2))

	local finalP = (yR*topP) + ((Vector(1)-yR)*bottomP)
	
	local topN = (xR*mesh.normals:getElement(0)) + ((Vector(1)-xR)*mesh.normals:getElement(1))
	local bottomN = (xR*mesh.normals:getElement(3)) + ((Vector(1)-xR)*mesh.normals:getElement(2))

	local finalN = (yR*topN) + ((Vector(1)-yR)*bottomN)	

	local out = {}
	out.point = finalP
	out.normal = finalN
	
	return out
	
end

function makeFur(meshIn, element, curves, min, max)
	
	if min == nil then
		min = 0.01
	end
	if max == nil then
		max = 0.05
	end	
	
	local mesh = {}
	mesh.points = meshIn:getPolygon(element) 
	mesh.normals = meshIn:getPolygonNormals(element)
	
	for i=1, 100 do
		local length = Vector(Rand(min, max))
		local blade = AttributeVector(0,4)
		local point = randomPointOnQuad(mesh, element)
		blade:setElement(0, point.point)
		blade:setElement(1, point.point)
		blade:setElement(2, point.point + point.normal * length )
		blade:setElement(3, point.point + point.normal * length )
		curves:addCurve(blade)
	end
	
end

------------------------------------------------------------------------------------------------------------------

Render(TransformPrim(Rotate(45,45,0) * Translate(0,0,10)))

local cube = Cube(10, 10, 10)
print("Created Cube.")

Render(cube)

cube:generateNormalsDirty();	
print("Generate Normals.")

local curves = Curves();

for i=0, (cube:getNPolys()):size()-1 do
	makeFur(cube, i, curves, 0.05, 0.2)
end

print("Fur Done. "..((cube:getNPolys()):size() * 100) )

curves:setWidth(Float(0.001))
Render(curves)

Render(cube)

output("RIB", "furCube.rib")


