#Modules needed my the current module. These can be thought of as includes and libraries in one thing.
import luyaODE as luya ; reload(luya)
from maya.mel import *
from mayaUtils import *
from luyaUtils import *

## Initialize our PD Controller
def init(g):
    #Initialise a new Physics World
    g.w = luya.World( ground=True )
    #Create a last Error variable to store the rate of change in Error
    g.lastErrorX = 0
    g.lastErrorY = 0
    g.lastErrorZ = 0
    
    del g.w.dynamics.CurrentBase.motor #By default motors are added to joints in luyaODE.
                                                                #For the purpose of this experiment we will remove them.

## Run Our PD Controller
def run(g):
    Current = Translate("CurrentTip") - Translate("CurrentBase")                #Retrieve the Current Direction of the Inverse Pendulum.
    Reference = Translate("ReferenceTip") - Translate("ReferenceBase")    #Retrieve the Direction of where we want to be.
    
    Axis = ( Reference ^ Current ).normal() # The Axis is the cross product of the Reference and Current vectors.
                                                                    # when Current moves to the other side of Reference the axis will flip,
                                                                    # this will flip the direction of rotation. A side effect of the design.
                                                                    # All we need to worry about is the AngularVelocity.
    Error = Current.angle( Reference )         # The current error. If this is 0 then we are in the right place.
      
    # Using a quaternion allows us to perform an neat Axis Angle rotation.
    EError = MQuaternion( Error, Axis ) . asEulerRotation()
    
    # Both of these constants have to be tweaked by hand
    kP = 20000      # The Proportional Constant
    kD = 1500000 # The Derivative Constant
    
    #Performing the PID Controll all all 3 Axis of Rotation
    Px = EError.x
    Dx = EError.x - g.lastErrorX
    SolutionX = kP*Px + kD*Dx
    
    Py = EError.y
    Dy = EError.y - g.lastErrorY
    SolutionY = kP*Py + kD*Dy
    
    Pz = EError.z
    Dz = EError.z - g.lastErrorZ
    SolutionZ = kP*Pz + kD*Dz
    
    #Apply the computed torque to the capsule body
    g.w.dynamics.CurrentTip.body.addTorque( (-SolutionX, -SolutionY, -SolutionZ) )
    
    #Increment the simulation timestep
    g.w.step( 0.01 )
    
    #Record the last Error
    g.lastErrorX = EError.x
    g.lastErrorY = EError.y
    g.lastErrorZ =EError.z

def end(g):
    pass
