#include<btWorld.h>
#include<cassert>

btBoxShape *btBoxShapeNew( btVector3 hwd ){
	return new btBoxShape( hwd ) ;
}

btWorld::btWorld(){
	dispatcher = new btCollisionDispatcher();
	broadphase = new btSimpleBroadphase(32766);
	solver = new btSequentialImpulseConstraintSolver;

	dynamicsWorld = new btDiscreteDynamicsWorld(dispatcher,broadphase,solver);
	dynamicsWorld->setGravity(btVector3(0,-9.8,0));
}

btWorld::btWorld( btVector3 gravity ){
	dispatcher = new btCollisionDispatcher();
	broadphase = new btSimpleBroadphase(32766);
	solver = new btSequentialImpulseConstraintSolver;

	dynamicsWorld = new btDiscreteDynamicsWorld(dispatcher,broadphase,solver);
	dynamicsWorld->setGravity(gravity);
}

btWorld::~btWorld(){
	for (int i=dynamicsWorld->getNumCollisionObjects()-1; i>=0 ;i--)
	{
		btCollisionObject* obj = dynamicsWorld->getCollisionObjectArray()[i];
		dynamicsWorld->removeCollisionObject( obj );
		delete obj;
	}
	int max = collisionShapes.size();
	for (int j=max-1;j>=0;j--)
	{
		btCollisionShape* shape = collisionShapes[j];
		delete shape;
	}
	if( max > 0){
		collisionShapes.clear();
	}

	//delete dynamics world
	delete dynamicsWorld;

	//delete solver
	delete solver;

	//delete broadphase
	delete broadphase;

	//delete dispatcher
	delete dispatcher;

}

btRigidBody *btWorld::RigidBody( btTransform startTransform, btCollisionShape *shape, double mass )
{
	//btBoxShape *shape = new btBoxShape( hwd );
	collisionShapes.push_back( shape );

	//rigidbody is dynamic if and only if mass is non zero, otherwise static
	bool isDynamic = (mass != 0.f);

	btVector3 localInertia(0,0,0);
	if (isDynamic)
		shape->calculateLocalInertia(mass,localInertia);

	btDefaultMotionState* myMotionState = new btDefaultMotionState(startTransform);
	btRigidBody* body = new btRigidBody(mass,myMotionState,shape,localInertia);
	dynamicsWorld->addRigidBody(body);
	
	return body ;
}

#if 0
void btWorld::step(){
	int speedup = 10 ;
	float ms = 1000000.f/60.f;
	if (dynamicsWorld){
		for( int i=0; i<speedup; ++i )
			dynamicsWorld->stepSimulation( 5.f );
	}
}
#endif

std::vector<btTransform> btWorld::getTransforms(){
	std::vector<btTransform> output ;
	btTransform m ;

	if (dynamicsWorld)
	{
		int numObjects = dynamicsWorld->getNumCollisionObjects();
		for (int i=0;i<numObjects;++i)
		{
			btCollisionObject* colObj = dynamicsWorld->getCollisionObjectArray()[i];
			btRigidBody* body = btRigidBody::upcast(colObj);

			if (body && body->getMotionState())
			{
				btDefaultMotionState* myMotionState = (btDefaultMotionState*)body->getMotionState();
				m = myMotionState->m_graphicsWorldTrans ;
			}else{
				m = colObj->getWorldTransform() ;
			}

			output.push_back( m );
		}
	}
	return output ;
}

btDynamicsWorld & btWorld::getWorld(){
	return *dynamicsWorld ;
}

