/* File : pyBullet */

%define MODULE_DOC
"
The Bullet Physics Engine.
"
%enddef

%module(docstring=MODULE_DOC) pyBullet
%{
#include "btWorld.h"
%}

// ------- SOME VECTOR DEFINITIONS
%include stl.i
/* Instantiate the required template specializations */
namespace std {
    %template(DoubleVector)					vector<double>;
    %template(TransformVector)				vector<btTransform>;
}

%include "bullet/BulletCollision/CollisionDispatch/btCollisionCreateFunc.h"

// ------- Wrapping btWorld
%feature("autodoc", "1") btWorld ;
%include "btWorld.h"

// ------- Wrapping the usefull Linear Math library functions
// ------- btScalar Wrapper
%feature("autodoc", "1") btScalar ;
%include "bullet/LinearMath/btScalar.h"

// ------- btQuadword Wrapper
%feature("autodoc", "1") btQuadWord ;
%extend btQuadWord {
  std::vector<double> getXYZ(){
  	std::vector<double> t ;
  	t.push_back( $self->getX() );
  	t.push_back( $self->getY() );
  	t.push_back( $self->getZ() );
  	return t ;
  }
}
%include "bullet/LinearMath/btQuadWord.h"

// ------- btVector Wrapper
%feature("autodoc", "1") btVector3 ;
%include "bullet/LinearMath/btVector3.h"
%include "bullet/LinearMath/btPoint3.h"

// ------- btQuaternion Wrapper
%feature("autodoc", "1") btQuaternion ;
%extend btQuaternion {
  std::vector<double> getXYZW(){
  	std::vector<double> t ;
  	t.push_back( $self->getX() );
  	t.push_back( $self->getY() );
  	t.push_back( $self->getZ() );
  	t.push_back( $self->getW() );
  	return t ;
  }
}
%include "bullet/LinearMath/btQuaternion.h"

// ------- btMatrix3x3 Wrapper
%feature("autodoc", "1") btMatrix3x3 ;
%extend btMatrix3x3 {
  std::vector<double> getMatrix3x3(){
  	std::vector<double> mV ;
  	btVector3 r0 = $self->getRow(0) ;
  	btVector3 r1 = $self->getRow(1) ;
  	btVector3 r2 = $self->getRow(2) ;
  	
  	mV.push_back( r0[0] );
  	mV.push_back( r0[1] );
  	mV.push_back( r0[2] );
  	
  	mV.push_back( r1[0] );
  	mV.push_back( r1[1] );
  	mV.push_back( r1[2] );
  	
  	mV.push_back( r2[0] );
  	mV.push_back( r2[1] );
  	mV.push_back( r2[2] );
  	
  	return mV ;
  }
}
%include "bullet/LinearMath/btMatrix3x3.h"

// ------- btTransform Wrapper
%feature("autodoc", "1") btTransform ;
%ignore btTransform::getOpenGLMatrix ;
%extend btTransform {
  std::vector<double> getMatrix4x4(){
  	std::vector<double> mV ;
  	btScalar m[16] ;
  	$self->getOpenGLMatrix( m ) ;
  	
  	mV.push_back(m[0]);
  	mV.push_back(m[1]);
  	mV.push_back(m[2]);
  	mV.push_back(m[3]);
  	
  	mV.push_back(m[4]);
  	mV.push_back(m[5]);
  	mV.push_back(m[6]);
  	mV.push_back(m[7]);
  	
  	mV.push_back(m[8]);
  	mV.push_back(m[9]);
  	mV.push_back(m[10]);
  	mV.push_back(m[11]);
  	
  	mV.push_back(m[12]);
  	mV.push_back(m[13]);
  	mV.push_back(m[14]);
  	mV.push_back(m[15]);
  	
  	return mV ;
  }
}
%include "bullet/LinearMath/btTransform.h"

// ------- The actual Dynamics Stuff

// ------- Get the 6Dof Constraint
%include "bullet/BulletDynamics/ConstraintSolver/btTypedConstraint.h"
%include "bullet/BulletDynamics/ConstraintSolver/btGeneric6DofConstraint.h"
%include "bullet/BulletDynamics/ConstraintSolver/btPoint2PointConstraint.h"
%include "bullet/BulletDynamics/ConstraintSolver/btHingeConstraint.h"

// ------- Get rigid bodies
%include "bullet/BulletCollision/CollisionDispatch/btCollisionObject.h"

%feature("autodoc", "1") btRigidBody ;
%include "bullet/BulletDynamics/Dynamics/btRigidBody.h"


// ------- Get collision shapes
%include "bullet/BulletCollision/CollisionShapes/btCollisionShape.h"
%include "bullet/BulletCollision/CollisionShapes/btConvexShape.h"
%include "bullet/BulletCollision/CollisionShapes/btMultiSphereShape.h"
%include "bullet/BulletCollision/CollisionShapes/btSphereShape.h"
%include "bullet/BulletCollision/CollisionShapes/btPolyhedralConvexShape.h"
%include "bullet/BulletCollision/CollisionShapes/btBoxShape.h"

//%include "bullet/BulletCollision/CollisionShapes/btCompoundShape.h"

%include "bullet/BulletCollision/CollisionDispatch/btCollisionDispatcher.h"
%include "bullet/BulletCollision/BroadphaseCollision/btOverlappingPairCache.h"
%include "bullet/BulletCollision/BroadphaseCollision/btSimpleBroadphase.h"
%include "bullet/BulletDynamics/ConstraintSolver/btSequentialImpulseConstraintSolver.h"

%include "bullet/LinearMath/btMotionState.h"
%include "bullet/LinearMath/btDefaultMotionState.h"

class btCollisionWorld {
	public:
		btCollisionWorld(btDispatcher* dispatcher,btOverlappingPairCache* pairCache, int stackSize = 2*1024*1024);
		int	getNumCollisionObjects();
		btCollisionObjectArray &getCollisionObjectArray ();
};

%ignore btCollisionWorld::rayTest() ;
%include "bullet/BulletCollision/CollisionDispatch/btCollisionWorld.h"


// ------- Get the Dynamics World
%include "bullet/BulletDynamics/Dynamics/btDynamicsWorld.h"
%include "bullet/BulletDynamics/Dynamics/btDiscreteDynamicsWorld.h"
