## Import the dynamics interface
import luyaODE as luya ; reload(luya)
## Import the fuzzyRules translater
import fuzzyRules as rules ; reload(rules)
## Import the Fuzzy Logic Module
from pyFuzzy import *
## Import the Mel Interface
from maya.mel import *
## Import the Luya Maya Utilities
from mayaUtils import *
## Import MVector and MQuaternion from the Maya C++ Interface
from maya.api import MVector,MQuaternion
## Grab some basic math utilities
from math import pi,degrees,radians
## Import the luya utility function
import luyaUtils as lu

halfPi = pi / 2

## Initialise our fuzzy feedback controller
def init( g ):
    """ The FUZZY Part """
    print "\n\n\n\n\n\n\n"
    
    ## Set up the Angle Linguistic Variable
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.Angle = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    ## Set up the Angular Velocity Linguistic Variable
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.AngularVelocity = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    ## Set up our Torque Linguistic Variable
    scale  = 180
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, 0, scale, scale*2 ])
    g.Torque = OutputVariable (u.lower,u.upper,(
        ("-High" , Linear(u(0),u(1)),  Linear(u(1),u(2))),
        ("Zero" , Linear(u(1)+scale/4,u(2)),  Linear(u(2),u(3)-scale/4)),
        ("+High" , Linear(u(2),u(3)),  Linear(u(3),u(4)))
    ))
    
    ## Interpret our ruleset and python function
    g.ruleSet = interpret(\
    """
        if Angle is -High and AngularVelocity is -High : Torque = +High
        if Angle is +High and AngularVelocity is +High: Torque = -High
        if Angle is +High and AngularVelocity is -High : Torque = Zero
        if Angle is -High and AngularVelocity is +High: Torque = Zero
    """
    )
    
   ### Set up our physics world
    g.w = luya.World( ground=True )
    
    g.lastError = 0 ## Record the last error so that we can compute the angular velocity

## Run fuzzy logic feedback controller
def run( g ):
    ## Housekeeping, Make local Declarations of our Linguistic variables for neatness
    Torque = g.Torque
    Angle = g.Angle
    AngularVelocity = g.AngularVelocity
    
   ## Grab transform information and turn it into something usable
    vThi = (Translate("CurrentTip") - Translate("CurrentBase")).normal()
    yAxis = (Translate("ReferenceTip") - Translate("ReferenceBase")).normal()
    zAxis = ( yAxis ^ MVector(0,1,0) ).normal()
    xAxis = ( zAxis ^ yAxis).normal()
    
    ## Work out the current error values
    Error = 90 - degrees( vThi.angle(xAxis) )
    ErrorChange = Error - g.lastError
    
    ## Fuzzify the error values
    Angle.fuzz(Error)
    AngularVelocity.fuzz(ErrorChange)
    
    ## Clean the output Variable
    Torque.begin()
    ## Evaluate the fuzzy rules
    exec(g.ruleSet)
    
   ## Defuzz the torque
    torque = Torque.defuzz()
    
    ## Convert from a quaternion to an Euler rotation. We lose information here.
    lEuler = MQuaternion(radians( torque ),zAxis).asEulerRotation()
    ## Because of the loss of information it is important to magnify the rotation by a P constant
    kP = -1
    
    ## Declare a local copy of motor for clarity
    motor = g.w.dynamics.CurrentBase.motor
    motor.setParam( luya.ParamVel,  lEuler.x*kP )
    motor.setParam( luya.ParamVel2, lEuler.y*kP )
    motor.setParam( luya.ParamVel3, lEuler.z*kP )
    
    ## Store the last error as a global
    g.lastError = Error
    g.w.step(0.1)

def end( g ):
   pass
