""" 
    All variables declared in this script are local to this script.
"""

import luyaUtils as lu
from fuzzyApi import *
import fuzzyApi
reload(fuzzyApi)
from fuzzyRules import *
import luyaODE as luya
from math import pi,degrees,radians
from maya.api import MVector,MQuaternion
from maya.mel import *

N= None
halfPi = pi / 2

def Translate(name,t=None):
    """Returns the global coordinates of a tranform at a Maya Vector"""
    if t == None:
        return MVector(xform(name,ws=N,q=N,t=N))
    else:
        xform(name,ws=N,t=(t[0],t[1],t[2]))

def makeMotor(w,knee):
    """ Given a joint makes a motor for it """
    parent = knee.getBody(0)
    child = knee.getBody(1)
    
    cap = luya.ode.AMotor(w.w)
    cap.attach(parent, child)
    
    cap.setMode(luya.ode.AMotorEuler)
    
    cap.setAxis( 0, 1, (1,0,0 ))
    cap.setAxis( 2, 2, (0,0,1 ))
    
    cap.setAngle(0,0)
    cap.setAngle(1,0)
    cap.setAngle(2,0)
    
    mstop = pi
    strength = 100000
    
    cap.setParam(luya.ParamFMax,strength)
    cap.setParam(luya.ParamLoStop,-mstop)
    cap.setParam(luya.ParamHiStop,mstop)
    
    cap.setParam(luya.ParamFMax2,strength)
    cap.setParam(luya.ParamLoStop2,-mstop)
    cap.setParam(luya.ParamHiStop2,mstop)
    
    cap.setParam(luya.ParamFMax3,strength)
    cap.setParam(luya.ParamLoStop3,-mstop)
    cap.setParam(luya.ParamHiStop3,mstop)
    
    return cap

def calcTorque(axis,angle,motor):
    lEuler = MQuaternion(angle,axis).asEulerRotation()
    strength = -100
    if True:
        #In the below example ODE velocities can only be set as Euler rotations.
        motor.setParam( luya.ParamVel,  lEuler.x*strength )
        motor.setParam( luya.ParamVel2, lEuler.y*strength )
        motor.setParam( luya.ParamVel3, lEuler.z*strength )
    else:
        motor.addTorques(lEuler.x*strength,lEuler.y*strength,lEuler.z*strength)

def init( g ):
    """ The FUZZY Part """
    print "\n\n\n\n\n\n\n"
    
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.Angle = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.AngularVelocity = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, 0, scale, scale*2 ])
    g.Torque = OutputVariable (u.lower,u.upper,(
        ("-High" , Linear(u(0),u(1)),  Linear(u(1),u(2))),
        ("Zero" , Linear(u(1),u(2)),  Linear(u(2),u(3))),
        ("+High" , Linear(u(2),u(3)),  Linear(u(3),u(4)))
    ))
    
    g.ruleSet = interpret(\
    """
        if Angle is -High and AngularVelocity is -High : Torque = +High
        if Angle is +High and AngularVelocity is +High: Torque = -High
        if Angle is +High and AngularVelocity is -High : Torque = Zero
        if Angle is -High and AngularVelocity is +High: Torque = Zero
    """
    )
    
    """ The Physics Part """
    w = luya.World()
    w.motors = luya.atom()
    w.motors.cap = makeMotor( w, w.dynamics.shin_knee.joint )
    
    g.w = w
    g.lastError = 0
    
    g.log = open("/bacva3/ltitley/Devel/Luya/fuzzy_logic/fuzzy.log","w")

def run( g ):
    """ Run fuzzy logic feedback controller """
    """ Grab the globals """
    time = g.time
    cap = g.w.motors.cap
    Torque = g.Torque
    ruleSet= g.ruleSet
    Angle = g.Angle
    AngularVelocity = g.AngularVelocity
    
    """Grab transform info and turn it into something usable"""
    vThi = (Translate("thi") - Translate("knee")).normal()
    vUp = (Translate("AimTip") - Translate("AimBase")).normal()
    vAxis = MVector(0,0,1) # (vUp ^ vThi).normal()
    vAcross = (vAxis ^ vUp).normal()
    
    """ Work out the current error values """
    Error = 90 - degrees( vThi.angle(vAcross) )
    ErrorChange = g.lastError - Error

    """Fuzzify the error values"""
    Angle.fuzz(Error)
    AngularVelocity.fuzz(ErrorChange)
    
    """Clean the output Variable"""
    Torque.begin()
    exec(ruleSet)
    
    """Defuzz the torque"""
    torque = Torque.defuzz(filename="/bacva3/ltitley/Devel/Luya/fuzzy_logic/fuzzframes/out_%s.xpm" % str(time).zfill(4) )
    
    g.log.write("Frame: %s Error: %s Solution: %s\n" % (g.time,Error,torque) )
    
    """Apply this to the motor"""
    Translate("ax",(vAxis*3)+Translate("knee"))
    setKeyframe("ax")
    
    calcTorque(vAxis,torque,cap)
    
    """Store the last error as a global"""
    g.lastError = Error
    
    g.w.step(0.01)

def end( g ):
    g.log.close()
    print "End"
