from maya.mel import *
from maya.api import MVector, MQuaternion, MEulerRotation
from mayaUtils import *

import luyaBullet as luya
from luyaBullet import *
reload(luya)

import luyaUtils as lu
from fuzzyApi import *
import fuzzyApi
reload(fuzzyApi)
from fuzzyRules import *
from math import pi,degrees,radians
halfPi = pi / 2

def end( g ):
    g.log.close()

def init(g):
    """ The Physics Part """
    g.w = luya.World()
    
    """ The FUZZY Part """
    print "\n\n\n\n\n\n\n"
    
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.Angle = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.AngularVelocity = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    scale  = 100
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, 0, scale, scale*2 ])
    g.Torque = OutputVariable (u.lower,u.upper,(
        ("-High" , Linear(u(0),u(1)),  Linear(u(1),u(2))),
        ("Zero" , Linear(u(1),u(2)),  Linear(u(2),u(3))),
        ("+High" , Linear(u(2),u(3)),  Linear(u(3),u(4)))
    ))
    
    g.ruleSet = interpret(\
    """
        if Angle is -High and AngularVelocity is -High : Torque = +High
        if Angle is +High and AngularVelocity is +High: Torque = -High
        if Angle is +High and AngularVelocity is -High : Torque = Zero
        if Angle is -High and AngularVelocity is +High: Torque = Zero
    """
    )

    g.lastError = 0
    g.log = open("/bacva3/ltitley/Devel/Luya/fuzzy_logic/fuzzy.txt","w")
    
    g.Thi = g.w.dynamics.shin_knee_thi.body

def run(g):
    """ Run fuzzy logic feedback controller """
    """ Grab the globals """
    time = g.time
    Torque = g.Torque
    ruleSet= g.ruleSet
    Angle = g.Angle
    AngularVelocity = g.AngularVelocity
    
    """Grab transform info and turn it into something usable"""
    vThi = (Translate("thi") - Translate("knee")).normal()
    vUp = (Translate("AimTip") - Translate("AimBase")).normal()
    vAxis = MVector(0,0,1) # (vUp ^ vThi).normal()
    vAcross = (vAxis ^ vUp).normal()
    
    """ Work out the current error values """
    Error = 90 - degrees( vThi.angle(vAcross) )
    ErrorChange = g.lastError - Error

    """Fuzzify the error values"""
    Angle.fuzz(Error)
    AngularVelocity.fuzz(ErrorChange)
    
    """Clean the output Variable"""
    Torque.begin()
    exec(ruleSet)
    
    """Defuzz the torque"""
    torque = Torque.defuzz(filename="/bacva3/ltitley/Devel/Luya/fuzzy_logic/fuzzframes/out_%s.xpm" % str(time).zfill(4) )
    
    #g.log.write( "Error: %s ErrorChange: %s Solution: %s\n" % (Error,ErrorChange,torque) )
    g.log.write( "Frame: %s Error: %s Solution: %s ErrorChange: %s\n" % (g.time,Error,torque,ErrorChange) )
    
    """Apply this to the motor"""
    Translate("ax",(vAxis*3)+Translate("knee"))
    setKeyframe("ax")
    
    """Store the last error as a global"""
    g.lastError = Error
    
    E = MQuaternion( torque, vAxis ).asEulerRotation()
    g.Thi.applyTorque( btVector3( E.x*-200, E.y*-200, E.z*-200 )  )
    
    g.w.step()
    
def end(g):
    del g.w
 