import ply.lex as lex
import ply.yacc as yacc

tokens = (	'ID', 'NUMBER',
			'IF', 'THEN', 'IS','ISNOT','AND', 'OR', 'NOT', 'ASSIGN'
)

literals = ['{','}','(',')',',']

reserved = {
	'if' : 'IF',
	'is' : 'IS',
	'isnot' : 'ISNOT',
	'and' : 'AND',
	'or' : 'OR',
	'not' : 'NOT'
}

t_ASSIGN = r'='
t_THEN = r':'
t_NUMBER = r'\d+\.*\d*'

def t_ID(t):
	r'\+*\-*[a-zA-Z_][a-zA-Z_0-9]*'
	t.type = reserved.get(t.value,'ID')    # Check for reserved words
	return t

def t_COMMENT(t):
    r'\#.*'
    pass

def t_newline(t):
	r'\n+'
	t.lexer.lineno += t.value.count("\n")

def t_error(t):
	print "Illegal character '%s'" % t.value[0]
	t.lexer.skip(1)

# Ignored characters
t_ignore = " \t"

# Build the lexer
lex.lex()

precedence = (
	('left','IF', 'THEN'),
	('left','ASSIGN'),
	('left', 'NOT'),
	('left', 'AND', 'OR'),
	('left', 'IS', 'ISNOT')
)

def p_block(p):
	"""block : rule
			 | block rule
	"""
	if len(p) > 2:
		p[0] = "%s\n%s" % (p[1],p[2])
	else:
		p[0] = p[1]

def p_rule(p):
	"""rule : IF expression THEN assignment
	"""
	p[0] = "f_%s( %s, [%s])" % (p[1],p[2],p[4])

def p_expression(p):
	"""expression : '(' expression ')'
				  | op
				  | factor
	"""
	if len(p) == 4 :
		p[0] = p[2]
	else:
		p[0] = p[1]

def p_assignment(p):
	"""assignment : factor ASSIGN factor
				  | assignment assignment
	"""
	if len(p) == 4:
		p[0] = '( %s, "%s" )' % (p[1],p[3])
	else:
		p[0] = "%s, %s" % (p[1],p[2])

def p_op(p):
	"""op : bop
		  | uop
		  | isnot
		  | is
	"""
	p[0] = p[1]

def p_bop(p):
	"""bop : expression AND expression
		   | expression OR expression
	"""
	p[0] = "f_%s( %s, %s) " % (p[2],p[1],p[3])

def p_isnot(p):
	"""isnot : expression ISNOT expression
	"""
	p[0] = 'f_not(f_is( %s, "%s")) ' % (p[1],p[3])

def p_is(p):
	"""is : expression IS expression
	"""
	p[0] = 'f_%s( %s, "%s") ' % (p[2],p[1],p[3])

def p_uop(p):
	"""uop : NOT expression
	"""
	p[0] = "f_%s( %s )" % ( p[1], p[2] )

def p_factor(p):
	"""factor : ID
			 | NUMBER
	"""
	p[0] = p[1]

# Error rule for syntax errors
def p_error(p):
	print "FUZZY RULES: Syntax error on line %s" % (p.lineno-1)

yacc.yacc()

def interpret(rules):
	'Interprets a set of fuzzy logic rules and turns them into python code'
	'Returns a string of the function names'
	return yacc.parse(rules)

