from math import sin,pi

Centroid = 1

def checkRange(lower,upper):
    "This function performs a check which raises the GreaterThanError exception"
    if lower > upper : raise GreaterThanError((lower,upper))

class Curve:
    def __init__(self,lower,upper):
        checkRange(lower,upper)
        self.lower = lower
        self.upper = upper

    def compute(self,value):
        return 0

class Singleton(Curve):
    def __init__(self,value):
        Curve.__dict__["__init__"](self,value,value)
    
    def compute( self, value ):
        if self.value == value:
            return 1.0
        else:
            return 0.0

class Linear(Curve):
    def __init__(self,lower,upper):
        Curve.__dict__["__init__"](self,lower,upper)
    
    def compute( self, value ):
        return (value - self.lower) / (self.upper - self.lower)

class Sin(Curve):
    def __init__(self,lower,upper):
        Curve.__dict__["__init__"](self,lower,upper)
    
    def compute( self, value ):
        l = (value - self.lower) / (self.upper - self.lower)
        return ((sin((l+pi/2)*pi)/2)+0.5)

class Set:
    def __init__(self,name,left,right):
        self.name = name
        checkRange(left.upper, right.lower)
        
        self.left = left
        self.right = right

def markPoint(buff,x,y):
    """Debug function,
        Draws a cross on the defussification buffer.
    """
    w = 2
    h = 2
    for i in range(x-w,x+w+1):
        buff[y][i] = 2
    for i in range(y-h,y+h+1):
        buff[i][x] = 2

def pixbuffToXPM(buff,width,height,filename="out.xpm"):
    output = open(filename,"w")
    outstr = \
"""
/* XPM */
static char out_xpm[] = { \"%s %s 3 1\",
                        "   c #FFFFFF",
                        ".  c #000000",
                        "+  c #FF0000"
""" % (width,height)

    for y in range(height):
        outstr += ',"'
        for x in range(width):
            j = buff[(y*-1)+height-1][x]
            if j == 0:
                outstr += " "
            elif j == 1:
                outstr += "."
            else:
                outstr += "+"
        outstr += '"\n'

    outstr += "};\n"

    output.write(outstr)
    output.close()

def f_and(m1,m2):
    """Fuzzy Intersection"""
    if m1 < m2:
        return m1
    else:
        return m2

def f_or(m1,m2):
    """Fuzzy Union"""
    if m1 > m2:
        return m1
    else:
        return m2

def f_not(m):
    """Fuzzy inversion"""
    return 1.0 - m

def f_getMembership(set,value):
    left = set.left
    right = set.right
    if value > left.lower and value < left.upper :
        "Are we on the left slope?"
        return set.left.compute( value )
    elif value > right.lower and value < right.upper :
        "Are we on the right slope?"
        return 1 - set.right.compute( value )
    elif value <= set.right.lower and value >= set.left.upper :
        "Are we in the middle"
        return 1.0
    else:
        "We are not even in the set"
        return 0.0

def defuzzCentroid( upper, lower, sets, values, rez=(100,50), filename=None ):
        "Specify what resolution the image should be"
        width = rez[0]
        height = rez[1]
        
        pixbuff = []
        
        "Get the variables range"
        vrange = upper-lower
        "Get the range of the image as a float to be used later"
        prange = float(width)
        
        """
            cX and cY are the coordinates of the centroid.
            sum will be used to compute the values of cX and cY.
        """
        cX = 0
        cY = 0
        sum = 0
        
        """Make the pixel buffer"""
        for i in range(height):
            pixbuff.append([])
            for j in range(width):
                pixbuff[i].append(0)
                
        """Compute the centroid"""
        for i in range(width):
            """
                Map the position of the the pixel with x coordinate i to a coordinate
                space of the output variable.
                k = i / prange gives a value along the width of the pixbuf from 0 to 1
                k*vrange + lower maps k to the output variable.
            """
            k = float(i)/prange
            j = k*vrange+lower
            del k

                
            """
                Compute the highest y coordinate for the coordinate j.
                The point on the curve.
            """
            y = 0
            "Loop through each set in the output variable"
            for k in sets.keys():
                """
                    Get the membership value of j in set k described by the curves
                    which define it.
                """
                ny = f_getMembership( sets[k], j )
                
                """
                    Check for membership value defined by fuzzy operations on the variable.
                    Compare the membership values and take the lowest. This does the chopping off
                    the top of the set.
                """
                if values.has_key(k):
                    ny = f_and(ny,values[k])
                    
                """
                    Compare the computed membership value with any from overlapping sets and take
                    the highest.
                """
                y = f_or(y,ny)

            y=int(y*height)
            for k in range(y):
                pixbuff[k][i] = 1
                cX += i
                cY += k
                sum+=1

        if sum == 0:
            cX = width / 2
            cY = height / 2
        else:
            cX /= sum
            cY /= sum
        
        """The two lines below are for debugging purposes only. They write the contents of the pixbuff to an xpm file."""
        if filename !=  None :
            markPoint(pixbuff,cX,cY)
            pixbuffToXPM(pixbuff,width,height,filename=filename)
        
        return ((float(cX) / prange) *vrange) + lower

def f_is( variable, setname ):
	"""A fuzzy comparison"""
	return variable.f_isEqualTo(setname)

def f_if( membershipValue, outputVariablesAndSets ):
	"""A fuzzy if statement"""
	for i in outputVariablesAndSets:
		i[0].f_assign(i[1],membershipValue)

class GreaterThanError(Exception):
	"""This exception should be raised when the user has provided two numeric values
	in which the first should be less than the second.
	For example when var1 > var2 the error is raised, but not for when var1 < var2.
	"""
	def __init__(self,values):
		self.values = values
	def __str__(self):
		return "%s > %s" % (values[0],values[1])

class Rules:
	def __init__(self,rules):
		pass

class InputVariable :
	def __init__(self, lower, upper, sets ):
		checkRange(lower,upper)
		
		self.sets = {}
		self.lower = float(lower)
		self.upper = float(upper)
		
		for i in sets:
			assert len(i) == 3, "There should be three components to each set tuple in the sets tuple."
			self.sets[i[0]] = Set( i[0], i[1], i[2] )

	def fuzz(self,number):
		lower = self.lower
		upper = self.upper
		number = float(number)
		if number < lower:
			number = lower
		elif number > upper:
			number = upper
		self.value = number
		
	def f_isEqualTo(self,setname):
		return f_getMembership( self.sets[setname], self.value )

class Stepper:
	"""Utility class to help with setting up a linguistic variable"""
	def __init__(self,lower,upper,steps):
		self.lower = float(lower)
		self.upper = float(upper)
		if type(steps) == type(0) or type(steps) == type(0.0):
			self.stepsize = (self.upper - self.lower ) / float(steps - 1 )
			self.steps = None
		else:
			self.steps = steps
			self.stepsize = None
	def step(self,num):
		if self.stepsize != None:
			return self.lower + ( float(num) * self.stepsize )
		else:
			return self.steps[num]
	def __call__(self,num):
		return self.step(num)

class OutputVariable(InputVariable):
	def __init__(self, lower, upper, sets ):
		InputVariable.__dict__["__init__"](self, lower, upper, sets )
		self.values = {}
		
		for i in sets:
			self.values[str(i[0])] = 0.0

	def begin(self):
		"""
			Reset the output variable's set's accumulative values to 0.0 .
		"""
		self.values = {}
		for i in self.sets.keys():
			self.values[i] = 0.0
		
	def defuzz(self,method=Centroid,rez=(300,150),filename=None):
		if method == Centroid:
			return defuzzCentroid( self.upper, self.lower, self.sets, self.values, rez, filename )

	def f_assign(self,setname,newValue):
		assert self.sets.has_key(setname), "Set '%s' does not exist, so you can't assign a value to it" % setname
		assert newValue >= 0 and newValue <= 1, "Membership values must be between 0 and 1"
		
		"""
		Take whichever value is greatest between newVale and the currentValue.
		currentValue is the last greatest membership value.
		If no membership value existed previously then take newValue.
		"""

		if self.values.has_key(setname):
			currentValue = self.values[setname]
		else:
			currentValue = newValue
		self.values[setname] = f_or(currentValue,newValue)
		#self.values[setname] = (currentValue+newValue)/2.0
