#Modules needed my the current module. These can be thought of as includes and libraries in one thing.
import luyaODE as luya ; reload(luya)
import fuzzyRules as rules ; reload(rules)
from pyFuzzy import *
from maya.mel import *
from mayaUtils import *
from luyaUtils import *

## Initialize our PD Controller
def init(g):
    #Initialise a new Physics World
    g.w = luya.World( ground=True )
    
    #Create a last Error variable to store the rate of change in Error
    g.lastErrorX = 0
    g.lastErrorY = 0
    g.lastErrorZ = 0
    
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.Angle = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    scale  = 90
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, scale, scale*2 ])
    g.AngularVelocity = InputVariable (u.lower,u.upper,(
        ("-High" , Singleton(u(0)),  Linear(u(1),u(2))),
        ("+High" , Linear(u(1),u(2)),  Singleton(u(3)))
    ))
    
    scale  = 100000
    u = Stepper(-scale*2,scale*2,[ -scale*2, -scale, 0, scale, scale*2 ])
    g.Torque = OutputVariable (u.lower,u.upper,(
        ("-High" , Linear(u(0),u(1)),  Linear(u(1),u(2))),
        ("Zero" , Linear(u(1)+scale/4,u(2)),  Linear(u(2),u(3)-scale/4)),
        ("+High" , Linear(u(2),u(3)),  Linear(u(3),u(4)))
    ))
    
    g.ruleSet = rules.interpret(\
    """
        if Angle is -High and AngularVelocity is -High : Torque = +High
        if Angle is +High and AngularVelocity is +High: Torque = -High
        if Angle is +High and AngularVelocity is -High : Torque = Zero
        if Angle is -High and AngularVelocity is +High: Torque = Zero
    """
    )
    
    #By default motors are added to joints in luyaODE.
    #For the purpose of this experiment we will remove them.
    del g.w.dynamics.CurrentBase.motor

## Run Our PD Controller
def run(g):
    Angle = g.Angle
    AngularVelocity = g.AngularVelocity
    Torque = g.Torque
    #Retrieve the Current Direction of the Inverse Pendulum.
    Current = Translate("CurrentTip") - Translate("CurrentBase")
    #Retrieve the Direction of where we want to be.
    Reference = Translate("ReferenceTip") - Translate("ReferenceBase")
    
    # The Axis is the cross product of the Reference and Current vectors.
    # when Current moves to the other side of Reference the axis will flip,
    # this will flip the direction of rotation. A side effect of the design.
    # All we need to worry about is the AngularVelocity.
    Axis = ( Reference ^ Current ).normal()
    
    # The current error. If this is 0 then we are in the right place.
    Error = Current.angle( Reference )
    
    # Using a quaternion allows us to perform a neat Axis Angle rotation.
    EError = MQuaternion( Error, Axis ) . asEulerRotation()
    
    print "%s %s %s" % ( EError.x, EError.y, EError.z )
    
    # Fuzzify the error values and compute the solution for X
    Angle.fuzz(EError.x)
    AngularVelocity.fuzz( EError.x - g.lastErrorX )
    Torque.begin() ; exec(g.ruleSet)
    SolutionX = Torque.defuzz()
    
    # Fuzzify the error values and compute the solution for Y
    Angle.fuzz(EError.y)
    AngularVelocity.fuzz( EError.y - g.lastErrorY )
    Torque.begin() ; exec(g.ruleSet)
    SolutionY = Torque.defuzz()
    
    # Fuzzify the error values and compute the solution for Z
    Angle.fuzz(EError.z)
    AngularVelocity.fuzz( EError.z - g.lastErrorZ )
    Torque.begin() ; exec(g.ruleSet)
    SolutionZ = Torque.defuzz()
    
    print "%s %s %s" % ( SolutionX, SolutionY, SolutionZ )
    #Apply the computed torque to the capsule body
    g.w.dynamics.CurrentTip.body.addTorque( (SolutionX, SolutionY, SolutionZ) )
    
    #Increment the simulation timestep
    g.w.step( 0.05 )
    
    #Record the last Error
    g.lastErrorX = EError.x
    g.lastErrorY = EError.y
    g.lastErrorZ =EError.z

def end(g):
    pass
