## @ingroup behaviours
# All variables declared in this script are local to this script.

import luyaUtils as lu
from fuzzyApi import *
from mayaUtils import *
import fuzzyApi
reload(fuzzyApi)
from fuzzyRules import *
import luyaODE as luya
reload(luya)
from math import pi,degrees,radians
from maya.api import MVector,MQuaternion
from maya.mel import *

import clearKeyframes
reload(clearKeyframes)

N= None
halfPi = pi / 2

def makeMotor(w,knee):
    """ Given a joint makes a motor for it """
    parent = knee.getBody(0)
    child = knee.getBody(1)
    
    cap = luya.ode.AMotor(w.w)
    cap.attach(parent, child)
    
    cap.setMode(luya.ode.AMotorEuler)
    
    cap.setAxis( 0, 1, (1,0,0 ))
    cap.setAxis( 2, 2, (0,0,1 ))
    
    cap.setAngle(0,0)
    cap.setAngle(1,0)
    cap.setAngle(2,0)
    
    mstop = pi
    strength = 100000
    
    cap.setParam(luya.ParamFMax,strength)
    #cap.setParam(luya.ParamLoStop,-mstop)
    #cap.setParam(luya.ParamHiStop,mstop)
    
    cap.setParam(luya.ParamFMax2,strength)
    cap.setParam(luya.ParamLoStop2,-mstop/200000)
    cap.setParam(luya.ParamHiStop2,mstop/200000)
    
    cap.setParam(luya.ParamFMax3,strength)
    #cap.setParam(luya.ParamLoStop3,-mstop)
    #cap.setParam(luya.ParamHiStop3,mstop)
    
    return cap

def calcTorque(axis,angle,motor):
    lEuler = MQuaternion(radians( angle ),axis).asEulerRotation()
    strength = -10
    #In the below example ODE velocities can only be set as Euler rotations.
    motor.setParam( luya.ParamVel,  lEuler.x*strength )
    motor.setParam( luya.ParamVel2, lEuler.y*strength )
    motor.setParam( luya.ParamVel3, lEuler.z*strength )

## @ingroup behaviours
# Basic jointTracker class
class jointTracker:
    def __init__( self, motor, strength=400 ):
        
        scale  = 90
        u = Stepper(-scale, scale,[ -2*scale, -scale, scale, 2*scale ])
        g.Angle = InputVariable (u.lower,u.upper,(
            ("Low" , Linear(u(0),u(1)-scale/10), Linear(u(1),u(2))),
            ("High", Linear(u(1)+scale/10,u(2)), Linear(u(2),u(3)))
        ))
        
        scale  = strength
        u = Stepper(-scale, scale,[ -2*scale, -scale, scale, 2*scale ])
        g.Torque = OutputVariable (u.lower,u.upper,(
            ("Low" , Linear(u(0),u(1)-scale/10), Linear(u(1),u(2))),
            ("High", Linear(u(1)+scale/10,u(2)), Linear(u(2),u(3)))
        ))
        
        g.ruleSet = interpret(\
        """
            if Angle is High : Torque = Low
            if Angle is Low : Torque = High
        """
        )

def run( g ):
    """ Run fuzzy logic feedback controller """
    """ Grab the globals """
    time = g.time
    cap = g.w.motors.cap
    Torque = g.Torque
    ruleSet= g.ruleSet
    Angle = g.Angle
    
    """Grab transform info and turn it into something usable"""
    vThi = (Translate("thi") - Translate("knee")).normal()
    yAxis = (Translate("AimTip") - Translate("AimBase")).normal()
    zAxis = ( yAxis ^ vThi ).normal()
    
    """ Work out the current error values """
    Error = degrees( vThi.angle(yAxis) )

    """Fuzzify the error values"""
    Angle.fuzz(Error)
    
    """Clean the output Variable"""
    Torque.begin()
    exec(ruleSet)
    
    """Defuzz the torque"""
    torque = Torque.defuzz(filename="/bacva3/ltitley/Devel/Luya/fuzzy_logic/fuzzframes/out_%s.xpm" % str(time).zfill(4) )
    
    g.log.write( "Frame: %s Error: %s Solution: %s\n" % (g.time,Error,torque) )
    
    """Apply this to the motor"""
    Translate("az",(zAxis*3)+Translate("knee"))
    setKeyframe("az")
    
    calcTorque(zAxis,torque,cap)
    
    """Store the last error as a global"""
    g.lastError = Error
    
    g.w.step(0.01)

def end( g ):
    g.log.close()
    print "End"
