/// A C++ Implimentation of part of the Python Fuzzy Logic library

#ifndef CFUZZY_
#define CFUZZY_

#include <boost/python.hpp>

#include <iostream>
#include <cmath>
#include <cassert>
#include <map>

#define len(T) ( extract<int>( T.attr("__len__")()) )

/// This function performs a check which raises the GreaterThanError exception
bool checkRange(double lower, double upper) ;

/// The Base Curve Class
class Curve {
	public:
	    Curve(double lower, double upper);
	    virtual double compute(double value) const { return 0.0 ; }  ;
	    
    	double m_lower ;
    	double m_upper ;
};

/// A Singleton
class Singleton : public Curve {
	public:
	    Singleton(double value);
	    double compute( double value ) const ;

    	double m_value ;
};

/// Linear
class Linear : public Curve {
	public:
	    Linear(double lower, double upper) ;
	    double compute( double value ) const ;
};

/// Half Sin wave
class Sin : public Curve {
	public:
	    Sin(double lower, double upper) ;
	    double compute( double value ) const ;
};

class Set {
	public:
	    Set(std::string name, Curve &left, Curve &right);
	    
    	std::string m_name	;
    	Curve *m_left		;
    	Curve *m_right		;
};

class InputVariable {
	public:
		InputVariable(double lower, double upper, boost::python::tuple sets );
		~InputVariable();
		void fuzz(double number);
		double f_isEqualTo(std::string setname);
		
		std::map< std::string, Set *> m_sets ;
		double m_lower;
		double m_upper;
		double m_value;
};

class OutputVariable : public InputVariable {
	public:
		OutputVariable( double lower, double upper, boost::python::tuple sets );
		///Reset the output variable's accumulative values to 0.0 .
		void begin();
		double defuzz( boost::python::tuple rez, std::string filename );
		void f_assign(std::string setname, double newValue);
		
		std::map< std::string, double > m_values ;
		
	private:
		double defuzzCentroid( boost::python::tuple rez, std::string filename );
};

///Mark a point on the pixbuff.
///For debugging purposes.
void markPoint(int *buff[], int x, int y);

///Draw the contents of the pixelbuffer to a file
///For debugging purposes.
void pixbuffToXPM(int *buff[], int width, int height, std::string filename );

///Fuzzy Intersection
double f_and( double m1, double m2);

///Fuzzy Union
double f_or( double m1, double m2);

///Fuzzy inversion
double f_not(double m);

///A Fuzzy comparison
double f_is( InputVariable *variable, std::string setname );

///A Fuzzy if statement
void f_if( double membershipValue, boost::python::list outputVariablesAndSets );

///Get the membership value of the variable of < variable > in the Set set.
double f_getMembership( Set &set, double value );

///Centroid deffuzzification
#if 0 
double defuzzCentroid(	double upper, double lower,
						boost::python::dict	sets,
						boost::python::dict	values,
						boost::python::tuple rez, std::string filename
					);
#endif
#endif
