/// A C++ Implimentation of part of the Python Fuzzy Logic library

#include<cFuzzy.h>

/// This function performs a check which raises the GreaterThanError exception
bool checkRange(double lower, double upper){
	if (lower <= upper ){
		return 1 ;
	}else{
		return 0 ;
	}
}

Curve::Curve(double lower, double upper){
    checkRange(lower,upper) ;
    m_lower = lower ;
    m_upper = upper ;
}

Singleton::Singleton(double value) : Curve( value, value ){
	m_value = value ;
}

double Singleton::compute( double value ) const {
    if (m_value == value) {
        return 1.0 ;
    } else {
        return 0.0 ;
    }
}


Linear::Linear(double lower, double upper) : Curve(lower, upper){
} ;

double Linear::compute( double value ) const {
    return (value - m_lower) / (m_upper - m_lower) ;
}



Sin::Sin(double lower, double upper) : Curve(lower, upper){
} ;

double Sin::compute( double value ) const {
    double l = (value - m_lower) / (m_upper - m_lower);
    return ((sin((l+M_PI/2)*M_PI)/2)+0.5);
}

Set::Set(std::string name, Curve &left, Curve &right){
    m_name = name ;
    checkRange(left.m_upper, right.m_lower);
    
    m_left = &left ;
    m_right = &right ;
}

///Mark a point on the pixbuff.
///For debugging purposes.
void markPoint(int *buff[], int x, int y){
	#if 0
    """Debug function,
        Draws a cross on the defussification buffer.
    """
    w = 2
    h = 2
    for i in range(x-w,x+w+1):
        buff[y][i] = 2
    for i in range(y-h,y+h+1):
        buff[i][x] = 2
    #endif
}

///Draw the contents of the pixelbuffer to a file
///For debugging purposes.
void pixbuffToXPM(int *buff[], int width, int height, std::string filename ){
	#if 0
    output = open(filename,"w")
    outstr = \
"""
/* XPM */
static char out_xpm[] = { \"%s %s 3 1\",
                        "   c #FFFFFF",
                        ".  c #000000",
                        "+  c #FF0000"
""" % (width,height)

    for y in range(height):
        outstr += ',"'
        for x in range(width):
            j = buff[(y*-1)+height-1][x]
            if j == 0:
                outstr += " "
            elif j == 1:
                outstr += "."
            else:
                outstr += "+"
        outstr += '"\n'

    outstr += "};\n"

    output.write(outstr)
    output.close()
    #endif
}

///Fuzzy Intersection
double f_and( double m1, double m2){
    if (m1 < m2){
        return m1 ;
	}else {
        return m2 ;
	}
}

///Fuzzy Union
double f_or( double m1, double m2){
    if (m1 > m2){
        return m1 ;
    }else {
        return m2 ;
    }
}

///Fuzzy inversion
double f_not(double m){
    return 1.0 - m ;
}

///Get the membership value of the variable of < variable > in the Set set.
double f_getMembership( Set &set, double value ){
    Curve* left = set.m_left ;
    Curve* right = set.m_right;
    
    if (value > left->m_lower and value < left->m_upper) {
        ///Are we on the left slope
        double v = left->compute( value ) ;
        return v ;
        
    } else if ( value > right->m_lower and value < right->m_upper) {
        ///Are we on the right slope?
        return 1 - right->compute( value ) ;
        
    } else if ( value <= right->m_lower and value >= left->m_upper) {
        ///Are we in the middle
        return 1.0 ;
        
    }else{
    	return 0.0 ;
    }
}

///Centroid deffuzzification
double defuzzCentroid(	double upper, double lower,
						std::map< std::string, Set *>	sets,
						std::map< std::string, double >	values,
						std::vector< int > rez, std::string filename
					){
        ///Specify what resolution the image should be
        int width = rez[0] ;
        int height = rez[1];
        
        double d_width = static_cast<double>(rez[0]) ;
        double d_height = static_cast<double>(rez[1]) ;
        
        int pixbuff[width][height] ;
        
        ///Get the variables range
        double vrange = upper-lower ;
        double prange = d_width ;
        
        ///    cX and cY are the coordinates of the centroid.
        ///   sum will be used to compute the values of cX and cY.
        int cX = 0	;
        int cY = 0	;
        double sum = 0	;
                
        ///Compute the centroid
        int i;
        for ( i=0; i<width; ++i ){
            
			//	Map the position of the the pixel with x coordinate i to a coordinate
			//	space of the output variable.
			//	k = i / prange gives a value along the width of the pixbuf from 0 to 1
			//	k*vrange + lower maps k to the output variable.

            double d_k = static_cast<double>(i)/prange ;
            double j = d_k * vrange + lower ;
                
			//	Compute the highest y coordinate for the coordinate j.
			//	The point on the curve.
            double d_y = 0 ;

            ///Loop through each set in the output variable
            std::map <std::string, Set*> ::iterator k = sets.begin() ;
			for ( ; k != sets.end() ; ++k ){
				//	Get the membership value of j in set k described by the curves
				//	which define it.
                double ny = f_getMembership( *sets[k->first], j ) ;
                
				//	Check for membership value defined by fuzzy operations on the variable.
				//	Compare the membership values and take the lowest. This does the chopping off
				//	the top of the set.
                if ( values.find(k->first) != values.end() ){
                    ny = f_and(ny,values[k->first]) ;
                }
                    
				//	Compare the computed membership value with any from overlapping sets and take
				//	the highest.
                d_y = f_or(d_y,ny) ;
			}

            int i_y = static_cast<int>( d_y * d_height ) ;
            for ( int m=0 ; m < i_y; ++m ){
                pixbuff[m][i] = 1 ;
                cX += i ;
                cY += m ;
                sum+=1 ;
            }
        }

        if ( sum == 0) {
            cX = width / 2  ;
            cY = height / 2 ;
        } else {
            cX /= static_cast<int>(sum) ;
            cY /= static_cast<int>(sum) ;
        }
        double d_cX = static_cast<double>(cX) ;
        
        ///The two lines below are for debugging purposes only. They write the contents of the pixbuff to an xpm file.
        if (filename.size() > 0 ){
            ///markPoint(pixbuff,cX,cY) ;
            ///pixbuffToXPM(pixbuff,width,height,filename=filename) ;
        }
        
        return ((d_cX / prange) *vrange) + lower ;
}
