#include <cFuzzy.h>
using namespace boost::python ;

BOOST_PYTHON_MODULE(cFuzzy)
{
	// Utility Functions
	def("checkRange", checkRange);
	def("f_and", f_and);
	def("f_or", f_or);
	def("f_not", f_not);
	def("f_if", f_if);
	def("f_is", f_is);
	def("f_getMembership", f_getMembership);
	
	/// Curve classes
	class_<Curve>("Curve", init<double,double>())
		.def_readonly("upper", &Curve::m_upper)
		.def_readonly("lower", &Curve::m_lower)
	;
	class_<Singleton, bases<Curve> >("Singleton",init<double>())
        .def("compute", &Singleton::compute)
    ;
    class_<Linear, bases<Curve> >("Linear",init<double,double>())
        .def("compute", &Linear::compute)
    ;
    class_<Sin, bases<Curve> >("Sin",init<double,double>())
        .def("compute", &Sin::compute)
    ;
    
    /// Set Class
    class_<Set>("Set", init<std::string , Curve &, Curve &>() )
        .def("compute", &Sin::compute)
        .def_readonly("name", &Set::m_name)
        .def_readonly("left", &Set::m_left)
        .def_readonly("right", &Set::m_right)
    ;
    
    /// InputVariable Class
    class_<InputVariable>("InputVariable", init< double, double, boost::python::tuple >() )
        .def_readonly("lower", &InputVariable::m_lower)
        .def_readonly("upper", &InputVariable::m_upper)
        .def("fuzz", &InputVariable::fuzz)
        .def("f_isEqualTo", &InputVariable::f_isEqualTo)
    ;
    
    /// OutputVariable Class
    class_<OutputVariable, bases<InputVariable> >("OutputVariable", init< double, double, boost::python::tuple >() )
        .def("fuzz", &OutputVariable::fuzz)
        .def("f_isEqualTo", &OutputVariable::f_isEqualTo)
       	.def("begin", &OutputVariable::begin)
       	.def("defuzz", &OutputVariable::defuzz)
       	.def("f_assign", &OutputVariable::f_assign)
    ;
}

